@@ -5,7 +5,7 @@ ConvertType is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2020, Free Software Foundation, Inc.
+Copyright (C) 2016-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -83,9 +83,11 @@ struct converttparams
   struct gal_options_common_params cp;   /* Common parameters.         */
   gal_list_str_t          *inputnames;   /* The names of input files.  */
   gal_list_str_t                *hdus;   /* The names of input hdus.   */
+  char           *globalhdu;  /* Global HDU (for all inputs).          */
   uint8_t           quality;  /* Quality of JPEG image.                */
   float           widthincm;  /* Width in centimeters.                 */
   uint32_t      borderwidth;  /* Width of border in PostScript points. */
+  uint8_t       bordercolor;  /* Color of the border.                  */
   uint8_t               hex;  /* Use hexadecimal not ASCII85 encoding. */
   gal_data_t      *colormap;  /* Color space to use for single/mono.   */
   uint8_t          rgbtohsv;  /* Convert input RGB channels to HSV.    */
@@ -97,6 +99,25 @@ struct converttparams
   char           *changestr;  /* String of change values.              */
   uint8_t  changeaftertrunc;  /* First convert, then truncate.         */
   uint8_t            invert;  /* ==1: invert the output image.         */
+  char           *marksname;  /* Filename with table with mark info.   */
+  char            *markshdu;  /* HDU of table with mark info.          */
+  char                *mode;  /* Mode of the coordinates for marks.    */
+  gal_list_str_t *markcoords; /* Coordinates of the marks.             */
+  gal_list_str_t  *marksize;  /* Columns containing mark size(s).      */
+  char       *marklinewidth;  /* Column containing mark line width.    */
+  char           *markcolor;  /* Column containing mark color.         */
+  char           *markshape;  /* Column containing mark shape.         */
+  char          *markrotate;  /* Column containing mark rotation.      */
+  char            *marktext;  /* Column containing mark text.          */
+  char            *markfont;  /* Column containing mark font name.     */
+  char        *markfontsize;  /* Column containing mark font size.     */
+  uint8_t        listcolors;  /* List available colors                 */
+  uint8_t         listfonts;  /* List available fonts.                 */
+  uint8_t         showfonts;  /* Show available fonts.                 */
+  uint8_t         sizeinpix;  /* Sizes are in pixels (in WCS-mode).    */
+  uint8_t      sizeinarcsec;  /* Sizes are in arcseconds (in WCS-mode).*/
+  uint8_t      sizeinarcmin;  /* Sizes are in arcminutes (in WCS-mode).*/
+  uint8_t marktextprecision;  /* Precision to convert floats.          */
 
   /* Internal */
   struct change     *change;  /* The value conversion string.          */
@@ -106,6 +127,7 @@ struct converttparams
   int             outformat;  /* The format of the output file.        */
   size_t              numch;  /* Current Channel.                      */
   gal_data_t          *chll;  /* Linked list of color channels.        */
+  gal_data_t         *marks;  /* Information of objects to show.       */
 };
 
 #endif
