@@ -5,7 +5,8 @@ Fits is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2021, Free Software Foundation, Inc.
+     Pedram Ashofteh-Ardakani <pedramardakani@pm.me>
+Copyright (C) 2016-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -110,6 +111,7 @@ ui_initialize_options(struct fitsparams *p,
 
   /* For clarity and non-zero initializations. */
   p->mode                = FITS_MODE_INVALID;
+  p->edgesampling        = GAL_BLANK_SIZE_T;
 
   /* Modify common options. */
   for(i=0; !gal_options_is_last(&cp->coptions[i]); ++i)
@@ -119,7 +121,6 @@ ui_initialize_options(struct fitsparams *p,
         {
         case GAL_OPTIONS_KEY_SEARCHIN:
         case GAL_OPTIONS_KEY_IGNORECASE:
-        case GAL_OPTIONS_KEY_TYPE:
         case GAL_OPTIONS_KEY_WCSLINEARMATRIX:
         case GAL_OPTIONS_KEY_DONTDELETE:
         case GAL_OPTIONS_KEY_LOG:
@@ -129,7 +130,7 @@ ui_initialize_options(struct fitsparams *p,
           break;
 
         case GAL_OPTIONS_KEY_OUTPUT:
-          cp->coptions[i].doc="Output file name (only for writing HDUs).";
+          cp->coptions[i].doc="Output file name (not for keywords).";
           break;
         }
 
@@ -268,10 +269,28 @@ ui_check_copykeys(struct fitsparams *p)
           break;
           */
         default:
-          error(EXIT_FAILURE, 0, "value to '--copykeys' must only contain "
-                "integer numbers and these special characters between them: "
-                "':' when necessary. But it is '%s' (the first "
-                "non-acceptable character is '%c').\n", p->copykeys, *pt);
+
+          /* If a name is given, it will start with an alphabetic
+             character, a hyphen, or an underscore (we already know that
+             the key doesn't start with a digit, since it would break out
+             of the 'switch'). Recall that according to the FITS standard,
+             a keyword name can only contain digits, alphabetic characters,
+             '-', or '_'. See "Section 4.1.2.1. Keyword name" of the FITS
+             4.0 standard.*/
+          if( isalpha(*pt) || *pt=='-' || *pt=='_' ) break;
+
+          /* Print an error if control reaches here because  */
+          error(EXIT_FAILURE, 0, "value to '--copykeys' must either be a "
+                "range of numbers (for example '--copykeys=10:20') or a "
+                "set of names (for example '--copykeys=KEY1,KEY2', for "
+                "any number of names, note that the first keyword name "
+                "should not contain only digits, thus get interpreted "
+                "as a number). However, your given value ('%s') couldn't "
+                "be interpreted as any of the two (the first "
+                "non-acceptable character is '%c'). If this seems to be "
+                "a problem on the programming side, please report the "
+                "issue to us at '%s'", p->copykeys, *pt,
+                PACKAGE_BUGREPORT);
           break;
         }
 
@@ -282,8 +301,19 @@ ui_check_copykeys(struct fitsparams *p)
       printf("\n\n------\n%c: %ld (%s)\n", *pt, read, tailptr);
       */
 
-      /* Make sure if a number was read at all? */
-      if(tailptr==pt) continue;   /* No number was read!                 */
+      /* If the value couldn't be read as a number, and the first element
+         hasn't been read as a number, parse it as a set of comma-separated
+         strings and break out of the parsing overall. */
+      if(tailptr==pt)
+        {
+          if(p->copykeysrange[0]==GAL_BLANK_LONG)
+            {
+              p->copykeysname=gal_options_parse_csv_strings_raw(pt,
+                                                                NULL, 0);
+              break;
+            }
+          else continue;
+        }
 
       /* Put it in the correct place. */
       p->copykeysrange[ forl=='f' ? 0 : 1 ]=read;
@@ -291,23 +321,38 @@ ui_check_copykeys(struct fitsparams *p)
     }
 
   /* Basic sanity checks. */
-  if( p->copykeysrange[1]==GAL_BLANK_LONG )
-    error(EXIT_FAILURE, 0, "no ending keyword number given to '--copykeys'. "
-          "If you want to copy all the keywords after a certain one "
-          "(without worrying about how many there are), you can use '-1'.\n\n"
-          "For example if you want to copy all the keywords after the 20th, "
-          "you can use '--copykeys=20,-1'. Generally, you can use negative "
-          "numbers for the last keyword number to count from the end.");
-  if( p->copykeysrange[0]<=0 )
-    error(EXIT_FAILURE, 0, "the first number given to '--copykeys' must be "
-          "positive");
-  if( p->copykeysrange[1]>=0 && p->copykeysrange[0]>=p->copykeysrange[1] )
-    error(EXIT_FAILURE, 0, "the first number (%ld) given to '--copykeys' "
-          "must be smaller than the second (%ld)", p->copykeysrange[0],
-          p->copykeysrange[1]);
+  if( p->copykeysname==NULL )
+    {
+      if( p->copykeysrange[1]==GAL_BLANK_LONG )
+        error(EXIT_FAILURE, 0, "no ending keyword number given to "
+              "'--copykeys'. If you want to copy all the keywords after "
+              "a certain one (without worrying about how many there "
+              "are), you can use '-1'.\n\n"
+              "For example if you want to copy all the keywords after "
+              "the 20th, you can use '--copykeys=20,-1'. Generally, "
+              "you can use negative numbers for the last keyword "
+              "number to count from the end.");
+      if( p->copykeysrange[0]<=0 )
+        error(EXIT_FAILURE, 0, "the first number given to '--copykeys' "
+              "must be positive");
+      if( p->copykeysrange[1]>=0
+          && p->copykeysrange[0]>=p->copykeysrange[1] )
+        error(EXIT_FAILURE, 0, "the first number (%ld) given to "
+              "'--copykeys' must be smaller than the second (%ld)",
+              p->copykeysrange[0], p->copykeysrange[1]);
+    }
 
   /* For a check:
-  printf("copykeys: %ld, %ld\n", p->copykeysrange[0], p->copykeysrange[1]);
+  if( p->copykeysname )
+    {
+      size_t i;
+      char **strarr=p->copykeysname->array;
+      printf("copykeys: ");
+      for(i=0;i<p->copykeysname->size;++i)
+        printf("%s ", strarr[i]);
+    }
+  else
+    printf("copykeys: %ld, %ld\n", p->copykeysrange[0], p->copykeysrange[1]);
   exit(0);
   */
 }
@@ -315,17 +360,55 @@ ui_check_copykeys(struct fitsparams *p)
 
 
 
-
-/* Read and check ONLY the options. When arguments are involved, do the
-   check in 'ui_check_options_and_arguments'. */
+/* If any of the META related keywords are given, set mode accordingly. */
 static void
-ui_read_check_only_options(struct fitsparams *p)
+ui_read_check_mode_meta(struct fitsparams *p)
+{
+  /* Make sure no other type of operation is requested. */
+  if(p->mode!=FITS_MODE_INVALID)
+    error(EXIT_FAILURE, 0, "meta image options (like '--pixelareaonwcs' "
+          "cannot be called with HDU or keyword manipulation options");
+
+  /* If '--type' is not given, inform the user. */
+  if(p->cp.type==0)
+    error(EXIT_FAILURE, 0, "no '--type' specified! This is necessary "
+          "to determine the numerical data type of the output image. "
+          "In most cases, '--type=float32' is sufficient");
+
+  /* If '--edgesampling' isn't given, inform the user. */
+  if(p->edgesampling==GAL_BLANK_SIZE_T)
+    error(EXIT_FAILURE, 0, "no '--edgesampling'! Please specify a "
+          "value to the '--edgesampling' option (usually '0' is "
+          "fine). This specifies the extra samplings along a pixel's "
+          "edge in options like '--pixelareaonwcs' in case "
+          "distortions are significant. Just note that non-zero "
+          "values will significantly slow down the program");
+
+  /* Check if a HDU is given. */
+  if(p->cp.hdu==NULL)
+    error(EXIT_FAILURE, 0, "a HDU (extension) is necessary for meta "
+          "image outputs (e.g., with '--pixelareaonwcs') but none "
+          "was defined. Please use the '--hdu' (or '-h') option to "
+          "select one of your input's HDUs. You can use the "
+          "'astfits input.fits' command to see the list of "
+          "available HDUs in 'input.fits'");
+
+  /* Set the mode. */
+  p->mode=FITS_MODE_META;
+
+}
+
+
+
+
+
+/* If any of the keyword manipulation options are requested, then set the
+   mode flag to keyword-mode. */
+static void
+ui_read_check_mode_keyword(struct fitsparams *p)
 {
   int checkkeys;
-  uint8_t stdoutcheck=0;
 
-  /* If any of the keyword manipulation options are requested, then set the
-     mode flag to keyword-mode. */
   if( p->date || p->comment || p->history || p->asis || p->keyvalue
       || p->delete || p->rename || p->update || p->write || p->verify
       || p->printallkeys || p->printkeynames || p->copykeys || p->datetosec
@@ -374,11 +457,21 @@ ui_read_check_only_options(struct fitsparams *p)
       /* Set the operating mode. */
       p->mode=FITS_MODE_KEY;
     }
+}
+
+
+
+
+/* Same for the extension-related options */
+static void
+ui_read_check_mode_extension(struct fitsparams *p)
+{
+  uint8_t stdoutcheck=0;
 
-  /* Same for the extension-related options */
   if( p->remove || p->copy || p->cut || p->numhdus || p->datasum
-      || p->pixelscale || p->skycoverage || p->hastablehdu
-      || p->hasimagehdu || p->listtablehdus || p->listimagehdus )
+      || p->pixelscale || p->pixelareaarcsec2 || p->skycoverage
+      || p->hastablehdu || p->hasimagehdu || p->listtablehdus
+      || p->listimagehdus )
     {
       /* A small sanity check. */
       if(p->mode!=FITS_MODE_INVALID)
@@ -387,8 +480,9 @@ ui_read_check_only_options(struct fitsparams *p)
 
       /* Some HDU options cannot be called with other options. */
       stdoutcheck = ( p->numhdus + p->datasum + p->pixelscale
-                      + p->skycoverage + p->hastablehdu + p->hasimagehdu
-                      + p->listtablehdus + p->listimagehdus );
+                      + p->pixelareaarcsec2 + p->skycoverage
+                      + p->hastablehdu + p->hasimagehdu + p->listtablehdus
+                      + p->listimagehdus );
 
       /* Make sure if an output file is needed. */
       if(stdoutcheck)
@@ -412,9 +506,10 @@ ui_read_check_only_options(struct fitsparams *p)
           if( ( p->numhdus || p->hastablehdu || p->hasimagehdu
                 || p->listtablehdus || p->listimagehdus)
               && p->cp.hdu==NULL )
-            error(EXIT_FAILURE, 0, "a HDU (extension) is necessary for the "
-                  "'--datasum', '--pixelscale' or '--skycoverage' options. "
-                  "Please use the '--hdu' (or '-h') option to select one");
+            error(EXIT_FAILURE, 0, "a HDU (extension) is necessary "
+                  "for the options that return information on one HDU "
+                  "(like ""'--pixelscale' or '--skycoverage'. Please "
+                  "use the '--hdu' (or '-h') option to select one");
         }
       else
         {
@@ -428,6 +523,21 @@ ui_read_check_only_options(struct fitsparams *p)
       /* Set the operating mode. */
       p->mode=FITS_MODE_HDU;
     }
+}
+
+
+
+
+
+/* Read and check ONLY the options. When arguments are involved, do the
+   check in 'ui_check_options_and_arguments'. */
+static void
+ui_read_check_only_options(struct fitsparams *p)
+{
+  /* Check the given optinos and set the operating mode accordingly. */
+  ui_read_check_mode_keyword(p);
+  ui_read_check_mode_extension(p);
+  if( p->pixelareaonwcs) ui_read_check_mode_meta(p);
 
   /* If no options are given, go into HDU mode, which will print the HDU
      information when nothing is asked. */
