@@ -5,7 +5,7 @@ Match is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2020, Free Software Foundation, Inc.
+Copyright (C) 2017-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -33,6 +33,8 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #define PROGRAM_EXEC   "astmatch" /* Program executable name. */
 #define PROGRAM_STRING PROGRAM_NAME" (" PACKAGE_NAME ") " PACKAGE_VERSION
 
+/* Internal constants */
+#define MATCH_KDTREE_ROOT_KEY "KDTROOT"
 
 
 enum match_modes
@@ -42,6 +44,14 @@ enum match_modes
   MATCH_MODE_CATALOG,
 };
 
+enum kdtree_modes
+{
+  MATCH_KDTREE_INVALID,           /* ==0 by default. */
+  MATCH_KDTREE_BUILD,
+  MATCH_KDTREE_INTERNAL,
+  MATCH_KDTREE_DISABLE,
+  MATCH_KDTREE_FILE,
+};
 
 
 
@@ -58,6 +68,8 @@ struct matchparams
   gal_data_t           *coord;  /* Array of manual coordinate values.   */
   gal_data_t         *outcols;  /* Array of second input column names.  */
   gal_data_t        *aperture;  /* Acceptable matching aperture.        */
+  char                *kdtree;  /* The mode to use k-d tree mode.       */
+  char             *kdtreehdu;  /* k-d tree HDU when its a (FITS) file. */
   uint8_t         logasoutput;  /* Don't rearrange inputs, out is log.  */
   uint8_t          notmatched;  /* Output is rows that don't match.     */
 
@@ -73,6 +85,9 @@ struct matchparams
   char              *out1name;  /* Name of first matched output.        */
   char              *out2name;  /* Name of second matched output.       */
   gal_list_str_t  *stdinlines;  /* Lines given by Standard input.       */
+  int              kdtreemode;  /* The k-d tree mode.                   */
+  gal_data_t      *kdtreedata;  /* The k-d tree data.                   */
+  size_t           kdtreeroot;  /* The root node of the k-d tree.       */
 
   /* Output: */
   time_t              rawtime;  /* Starting time of the program.        */
