@@ -5,7 +5,7 @@ Match is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2021, Free Software Foundation, Inc.
+Copyright (C) 2017-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -26,8 +26,10 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <errno.h>
 #include <error.h>
 #include <stdio.h>
+#include <string.h>
 
 #include <gnuastro/fits.h>
+#include <gnuastro/threads.h>
 
 #include <gnuastro-internal/timing.h>
 #include <gnuastro-internal/options.h>
@@ -106,6 +108,7 @@ ui_initialize_options(struct matchparams *p,
   cp->program_exec       = PROGRAM_EXEC;
   cp->program_bibtex     = PROGRAM_BIBTEX;
   cp->program_authors    = PROGRAM_AUTHORS;
+  cp->numthreads         = gal_threads_number();
   cp->coptions           = gal_commonopts_options;
 
 
@@ -119,7 +122,6 @@ ui_initialize_options(struct matchparams *p,
           cp->coptions[i].doc="Extension name or number of first input.";
           break;
         case GAL_OPTIONS_KEY_TYPE:
-        case GAL_OPTIONS_KEY_NUMTHREADS:
           cp->coptions[i].flags=OPTION_HIDDEN;
           break;
         }
@@ -163,19 +165,21 @@ parse_opt(int key, char *arg, struct argp_state *state)
   /* Set the key to this option. */
   switch(key)
     {
-
-    /* Read the non-option tokens (arguments): */
+    /* Read the non-option tokens (arguments). */
     case ARGP_KEY_ARG:
+      /* The user may give a shell variable that is empty! In that case
+         'arg' will be an empty string! We don't want to account for such
+         cases (and give a clear error that no input has been given). */
       if(p->input1name)
         {
           if(p->input2name)
-            argp_error(state, "only two arguments (input files) should be "
-                       "given, not any more");
+            argp_error(state, "only two arguments (input files) "
+                       "should be given, not any more");
           else
-            p->input2name=arg;
+            { if(arg[0]!='\0') p->input2name=arg; }
         }
       else
-        p->input1name=arg;
+        { if(arg[0]!='\0') p->input1name=arg; }
       break;
 
 
@@ -214,10 +218,42 @@ parse_opt(int key, char *arg, struct argp_state *state)
 static void
 ui_read_check_only_options(struct matchparams *p)
 {
-  if(p->outcols && p->notmatched)
-    error(EXIT_FAILURE, 0, "'--outcols' and '--notmatched' cannot be called "
-          "at the same time. The former is only for cases when the matches "
-          "are required");
+  /* k-d tree specific sanity checks. */
+  if(p->kdtree)
+  {
+    /* Set the k-d tree mode. */
+    if(      !strcmp(p->kdtree,"build")    ) p->kdtreemode=MATCH_KDTREE_BUILD;
+    else if( !strcmp(p->kdtree,"internal") ) p->kdtreemode=MATCH_KDTREE_INTERNAL;
+    else if( !strcmp(p->kdtree,"disable")  ) p->kdtreemode=MATCH_KDTREE_DISABLE;
+    else if( gal_fits_name_is_fits(p->kdtree) ) p->kdtreemode=MATCH_KDTREE_FILE;
+    else
+      error(EXIT_FAILURE, 0, "'%s' is not valid for '--kdtree'. The "
+            "following values are accepted: 'build' (to build the k-d tree in "
+            "the file given to '--output'), 'internal' (to force internal "
+            "usage of a k-d tree for the matching), 'disable' (to not use a "
+            "k-d tree at all), a FITS file name (the file to read a created "
+            "k-d tree from)", p->kdtree);
+
+    /* Make sure that the k-d tree build mode is not called with
+       '--outcols'. */
+    if( p->kdtreemode==MATCH_KDTREE_BUILD && (p->outcols || p->coord) )
+      error(EXIT_FAILURE, 0, "the '--kdtree=build' option is incompatible "
+            "with the '--outcols' or '--coord' options (because in the k-d "
+            "tree building mode doesn't involve actual matching. It will "
+            "only build k-d tree and write it to a file so it can be used "
+            "in future matches)");
+
+    /* Make sure that a HDU is also specified for the k-d tree when its an
+       external file. */
+    if( p->kdtreemode==MATCH_KDTREE_FILE && p->kdtreehdu==NULL )
+      error(EXIT_FAILURE, 0, "no '--kdtreehdu' specified! When a FITS "
+            "file is given to '--kdtree', you should specify its "
+            "extension/HDU within the file using '--kdtreehdu'. You "
+            "can either use the HDU name, or its number (counting "
+            "from 0). If you aren't sure what HDUs exist in the file, "
+            "you can use the 'astfits %s' command to see the full list",
+            p->kdtree);
+  }
 }
 
 
@@ -230,21 +266,28 @@ ui_read_check_only_options(struct matchparams *p)
 static void
 ui_check_options_and_arguments(struct matchparams *p)
 {
-  /* When '--coord' is given, there should be no second catalog
-     (argument). */
-  if(p->coord)
+  /* When '--coord' is given, or we are in k-d tree building mode, there
+     should be no second catalog (argument). */
+  if(p->coord || p->kdtreemode==MATCH_KDTREE_BUILD)
     {
       /* Make sure no second argument is given. */
       if(p->input2name)
         error(EXIT_FAILURE, 0, "only one argument can be given with the "
-              "'--coord' option");
+              "'--coord' or '--kdtree=build' options");
 
-      /* No need for 'p->input2name' or 'p->ccol2'. */
-      gal_data_free(p->ccol2);
-      p->ccol2=NULL;
+      /* In case '--ccol2' is given. */
+      if(p->ccol2 && p->cp.quiet==0)
+        {
+          /* Print a warning to let the user know the option will not be
+             used (the user had probably confused things if they have given
+             it). */
+          error(EXIT_SUCCESS, 0, "WARNING: with '--coord' or '--kdtree', "
+                "you only need to set '--ccol1' (since there is only a "
+                "single input table), '--ccol2' is not needed");
+        }
     }
 
-  /* '--coord' is not given. */
+  /* We need two input catalogs. */
   else
     {
       /* Without 'coord' atleast one input is necessary. */
@@ -264,10 +307,9 @@ ui_check_options_and_arguments(struct matchparams *p)
         }
     }
 
-
   /* If the first input is a FITS file, make sure its HDU is also given. */
   if( p->input1name
-      && gal_fits_name_is_fits(p->input1name)
+      && gal_fits_file_recognized(p->input1name)
       && p->cp.hdu==NULL )
     error(EXIT_FAILURE, 0, "no HDU for first input. When the input is "
           "a FITS file, a HDU must also be specified, you can use the "
@@ -275,7 +317,7 @@ ui_check_options_and_arguments(struct matchparams *p)
           "from zero), extension name, or anything acceptable by "
           "CFITSIO");
   if( p->input2name
-      && gal_fits_name_is_fits(p->input2name)
+      && gal_fits_file_recognized(p->input2name)
       && p->hdu2==NULL )
     error(EXIT_FAILURE, 0, "no HDU for second input. Please use "
           "the '--hdu2' ('-H') option and give it the HDU number "
@@ -316,7 +358,7 @@ ui_set_mode(struct matchparams *p)
      'NULL'), then we go into catalog mode because currently we assume
      standard input is only for plain text and WCS matching is not defined
      on plain text. */
-  if( p->input1name && gal_fits_name_is_fits(p->input1name) )
+  if( p->input1name && gal_fits_file_recognized(p->input1name) )
     {
       tin1=gal_fits_hdu_format(p->input1name, p->cp.hdu);
       p->mode = (tin1 == IMAGE_HDU) ? MATCH_MODE_WCS : MATCH_MODE_CATALOG;
@@ -340,7 +382,7 @@ ui_set_mode(struct matchparams *p)
      file.*/
   if(p->input2name)
     {
-      if(gal_fits_name_is_fits(p->input2name))
+      if(gal_fits_file_recognized(p->input2name))
         {
           tin2=gal_fits_hdu_format(p->input2name, p->hdu2);
           if(tin1==IMAGE_HDU && tin2!=IMAGE_HDU)
@@ -580,10 +622,10 @@ ui_read_columns_aperture_3d(struct matchparams *p)
 static size_t
 ui_set_columns_sanity_check_read_aperture(struct matchparams *p)
 {
-  size_t ccol1n, ccol2n;
+  size_t ccol1n=0, ccol2n=0;
 
-  /* Make sure the columns to read are given. */
-  if(p->coord)
+  /* Make sure the columns to match are given. */
+  if(p->coord || p->kdtreemode==MATCH_KDTREE_BUILD)
     {
       if(p->ccol1==NULL)
         error(EXIT_FAILURE, 0, "no value given to '--ccol1' (necessary with "
@@ -596,17 +638,22 @@ ui_set_columns_sanity_check_read_aperture(struct matchparams *p)
               "They specify the columns containing the coordinates to match");
     }
 
-  /* Make sure the same number of columns is given to both. */
+  /* Make sure the same number of columns is given to both. Note that a
+     second catalog is only necessary when we aren't building a k-d
+     tree. */
   ccol1n = p->ccol1->size;
-  ccol2n = p->coord ? p->coord->size : p->ccol2->size;
-  if(ccol1n!=ccol2n)
-    error(EXIT_FAILURE, 0, "number of coordinates given to '--ccol1' "
-          "(%zu) and '--%s' (%zu) must be equal.\n\n"
-          "If you didn't call these options, run with '--checkconfig' to "
-          "see which configuration file is responsible. You can always "
-          "override the configuration file values by calling the option "
-          "manually on the command-line",
-          ccol1n, p->coord ? "coord" : "ccol2", ccol2n);
+  if( p->kdtreemode!=MATCH_KDTREE_BUILD )
+    {
+      ccol2n = p->coord ? p->coord->size : p->ccol2->size;
+      if(ccol1n!=ccol2n)
+        error(EXIT_FAILURE, 0, "number of coordinates given to '--ccol1' "
+              "(%zu) and '--%s' (%zu) must be equal.\n\n"
+              "If you didn't call these options, run with '--checkconfig' "
+              "to see which configuration file is responsible. You can "
+              "always override the configuration file values by calling "
+              "the option manually on the command-line", ccol1n,
+              p->coord ? "coord" : "ccol2", ccol2n);
+    }
 
   /* Read/check the aperture values. */
   if(p->aperture)
@@ -627,7 +674,7 @@ ui_set_columns_sanity_check_read_aperture(struct matchparams *p)
               "dimensions is deduced from the number of values given to "
               "'--ccol1' (or '--coord') and '--ccol2'", ccol1n);
       }
-  else
+  else if ( p->kdtreemode != MATCH_KDTREE_BUILD )
     error(EXIT_FAILURE, 0, "no matching aperture specified. Please use "
           "the '--aperture' option to define the acceptable aperture for "
           "matching the coordinates (in the same units as each "
@@ -691,8 +738,8 @@ ui_read_columns_to_double(struct matchparams *p, char *filename, char *hdu,
     p->stdinlines=gal_options_check_stdin(filename, p->cp.stdintimeout,
                                           "input");
   tout=gal_table_read(filename, hdu, filename ? NULL : p->stdinlines,
-                      cols, cp->searchin, cp->ignorecase, cp->minmapsize,
-                      p->cp.quietmmap, NULL);
+                      cols, cp->searchin, cp->ignorecase, cp->numthreads,
+                      cp->minmapsize, p->cp.quietmmap, NULL);
 
   /* A small sanity check. */
   if(gal_list_data_number(tout)!=numcols)
@@ -729,6 +776,71 @@ ui_read_columns_to_double(struct matchparams *p, char *filename, char *hdu,
 
 
 
+#define UI_READ_KDTREE_FIXED_MSG "You can construct a k-d tree "        \
+  "for your first input table using the command below (just set your "  \
+  "desired output name, and give that file to '--kdtree', in a later "  \
+  "call with the same first input catalog):\n\n"                  \
+  "    astmatch %s -h%s --ccol1=%s,%s --kdtree=build --output=KDTREE.fits\n\n" \
+  "To learn more about the expected format of k-d trees in Gnuastro, "  \
+  "please run this command: 'info gnuastro k-d'"
+
+static void
+ui_read_kdtree(struct matchparams *p)
+{
+  size_t *sizetarr;
+  char **strarr1 = p->ccol1->array;
+  gal_data_t *keysll=gal_data_array_calloc(1);
+
+  /* Read the external k-d tree. */
+  p->kdtreedata=gal_table_read(p->kdtree, p->kdtreehdu, NULL,
+                               NULL, GAL_TABLE_SEARCH_NAME, 1,
+                               p->cp.numthreads, p->cp.minmapsize,
+                               p->cp.quietmmap, NULL);
+
+  /* It has to have only two columns, with an unsigned 32-bit integer
+     type in each. */
+  if( gal_list_data_number(p->kdtreedata)!=2 )
+    error(EXIT_FAILURE, 0, "%s (hdu: %s, that was given to "
+          "'--kdtree') has %zu column(s), but should contain 2 "
+          "columns. " UI_READ_KDTREE_FIXED_MSG,
+          p->kdtree, p->kdtreehdu, gal_list_data_number(p->kdtreedata),
+          p->input1name, p->cp.hdu, strarr1[0], strarr1[1]);
+  if( p->kdtreedata->type!=GAL_TYPE_UINT32
+      || p->kdtreedata->next->type!=GAL_TYPE_UINT32 )
+    error(EXIT_FAILURE, 0, "%s (hdu: %s, that was given to "
+          "'--kdtree') doesn't have unsigned 32-bit integer columns. "
+          UI_READ_KDTREE_FIXED_MSG, p->kdtree, p->kdtreehdu,
+          p->input1name, p->cp.hdu, strarr1[0], strarr1[1]);
+  if( p->kdtreedata->size!=p->cols1->size )
+    error(EXIT_FAILURE, 0, "%s (hdu: %s, that was given to "
+          "'--kdtree') doesn't have the same number of rows as "
+          "%s, which is the first given input. "
+          UI_READ_KDTREE_FIXED_MSG, p->kdtree, p->kdtreehdu,
+          gal_fits_name_save_as_string(p->input1name, p->cp.hdu),
+          p->input1name, p->cp.hdu, strarr1[0], strarr1[1]);
+
+  /* Read the k-d tree root. */
+  keysll[0].type=GAL_TYPE_SIZE_T;
+  keysll[0].name=MATCH_KDTREE_ROOT_KEY;
+  gal_fits_key_read(p->kdtree, p->kdtreehdu, keysll, 0, 0);
+  if(keysll[0].status)
+    error(EXIT_FAILURE, 0, "%s (hdu: %s, that was given to "
+          "'--kdtree') doesn't have the '%s' keyword, or it "
+          "couldn't be read as a number. " UI_READ_KDTREE_FIXED_MSG,
+          p->kdtree, p->kdtreehdu, MATCH_KDTREE_ROOT_KEY,
+          p->input1name, p->cp.hdu, strarr1[0], strarr1[1]);
+  sizetarr=keysll[0].array;
+  p->kdtreeroot=sizetarr[0];
+
+  /* Clean up: since the 'name' component wasn't allocated in this example,
+     we should set it to NULL before calling 'gal_data_array_free'. */
+  keysll[0].name=NULL;
+  gal_data_array_free(keysll, 1, 1);
+}
+
+
+
+
 
 /* Read catalog columns */
 static void
@@ -743,8 +855,10 @@ ui_read_columns(struct matchparams *p)
 
   /* Convert the array of strings to a list of strings for the column
      names. */
-  strarr1=p->ccol1->array;
-  strarr2=p->coord?NULL:p->ccol2->array;
+  strarr1 = p->ccol1->array;
+  strarr2 = ( (p->coord || p->kdtreemode==MATCH_KDTREE_BUILD)
+              ? NULL
+              : p->ccol2->array );
   for(i=0;i<ndim;++i)
     {
       gal_list_str_add(&cols1, strarr1[i], 1);
@@ -756,10 +870,32 @@ ui_read_columns(struct matchparams *p)
   /* Read-in the columns. */
   p->cols1=ui_read_columns_to_double(p, p->input1name, p->cp.hdu,
                                      cols1, ndim);
-  p->cols2=( p->coord
-             ? ui_set_columns_from_coord(p)
-             : ui_read_columns_to_double(p, p->input2name, p->hdu2,
-                                         cols2, ndim) );
+  if( p->kdtreemode!=MATCH_KDTREE_BUILD )
+    p->cols2=( p->coord
+               ? ui_set_columns_from_coord(p)
+               : ui_read_columns_to_double(p, p->input2name, p->hdu2,
+                                           cols2, ndim) );
+
+  /* If an external k-d tree is given, read it and make sure it has the
+     same number of rows as the first input and the proper datatype. */
+  if( p->kdtreemode==MATCH_KDTREE_FILE )
+    ui_read_kdtree(p);
+
+  /* If we are in k-d tree based matching and the second dataset is smaller
+     than the first, print a warning to let the user know that the speed
+     can be greatly improved if they swap the two. */
+  if( !p->cp.quiet
+      && p->kdtreemode!=MATCH_KDTREE_BUILD
+      && p->kdtreemode!=MATCH_KDTREE_DISABLE
+      && p->cols1->size > (2*p->cols2->size) )
+    error(EXIT_SUCCESS, 0, "TIP: the matching speed will GREATLY IMPROVE "
+          "if you swap the two inputs. Currently the second input has "
+          "fewer rows than the first. In the k-d tree based matching, "
+          "multi-threading will occur on the second input's rows and "
+          "the k-d will be constructed for the first input. Fewer "
+          "first-input rows therefore means a smaller tree, and more "
+          "second-input rows will be better distributed in your "
+          "system's CPU threads");
 
   /* Free the extra spaces. */
   gal_list_str_free(cols1, 1);
@@ -784,56 +920,71 @@ ui_preparations_out_cols(struct matchparams *p)
      proper list. */
   for(i=0;i<p->outcols->size;++i)
     {
+      /* For easy reading. */
       col=strarr[i];
-      switch(col[0])
+
+      /* In no-match mode, then the same column will be used from both
+         catalogs so things are easier. */
+      if(p->notmatched)
         {
-        case 'a': gal_list_str_add(&p->acols, col+1, 0); break;
-        case 'b':
-          /* With '--coord', only numbers that are smaller than the number
-             of the dimensions are acceptable. */
-          if(p->coord)
-            {
-              goodvalue=0;
-              rptr=gal_type_string_to_number(col+1, &readtype);
-              if(rptr)
-                {
-                  read=gal_data_alloc(rptr, readtype, 1, &one, NULL, 0, -1,
-                                      1, NULL, NULL, NULL);
-                  if(gal_type_is_int(readtype))
-                    {
-                      read=gal_data_copy_to_new_type_free(read,GAL_TYPE_LONG);
-                      if( *((long *)(read->array)) <= ndim )
-                        goodvalue=1;
-                    }
-                  gal_data_free(read);
-                }
-              if(goodvalue==0)
-                error(EXIT_FAILURE, 0, "bad value to second catalog "
-                      "column (%s) of '--outcols'.\n\n"
-                      "With the '--coord' option, the second catalog is "
-                      "assumed to have a single row and the given number "
-                      "of columns. Therefore when using '--outcols', only "
-                      "integers that are less than the number of "
-                      "dimensions (%zu in this case) are acceptable", col+1,
-                      ndim);
-            }
-          gal_list_str_add(&p->bcols, col+1, 0);
-          break;
-        default:
-          error(EXIT_FAILURE, 0, "'%s' is not a valid value for "
-                "'--outcols'.\n\n"
-                "The first character of each value to this option must be "
-                "either 'a' or 'b'. The former specifies a column from the "
-                "first input and the latter a column from the second. The "
-                "characters after them can be any column identifier (number, "
-                "name, or regular expression). For more on column selection, "
-                "please run this command:\n\n"
-                "    $ info gnuastro \"Selecting table columns\"\n",
-                col);
+          gal_list_str_add(&p->acols, col, 0);
+          gal_list_str_add(&p->bcols, col, 0);
         }
+
+      /* In match mode, we need to know which column should come from which
+         catalog. */
+      else
+        switch(col[0])
+          {
+          case 'a': gal_list_str_add(&p->acols, col+1, 0); break;
+          case 'b':
+            /* With '--coord', only numbers that are smaller than the
+               number of the dimensions are acceptable. */
+            if(p->coord)
+              {
+                goodvalue=0;
+                rptr=gal_type_string_to_number(col+1, &readtype);
+                if(rptr)
+                  {
+                    read=gal_data_alloc(rptr, readtype, 1, &one, NULL, 0,
+                                        -1, 1, NULL, NULL, NULL);
+                    if(gal_type_is_int(readtype))
+                      {
+                        read=gal_data_copy_to_new_type_free(read,
+                                                            GAL_TYPE_LONG);
+                        if( *((long *)(read->array)) <= ndim )
+                          goodvalue=1;
+                      }
+                    gal_data_free(read);
+                  }
+                if(goodvalue==0)
+                  error(EXIT_FAILURE, 0, "bad value to second catalog "
+                        "column (%s) of '--outcols'.\n\n"
+                        "With the '--coord' option, the second catalog "
+                        "is assumed to have a single row and the given "
+                        "number of columns. Therefore when using "
+                        "'--outcols', only integers that are less than "
+                        "the number of dimensions (%zu in this case) "
+                        "are acceptable", col+1, ndim);
+              }
+            gal_list_str_add(&p->bcols, col+1, 0);
+            break;
+          default:
+            error(EXIT_FAILURE, 0, "'%s' is not a valid value for "
+                  "'--outcols'.\n\n"
+                  "The first character of each value to this option "
+                  "must be either 'a' or 'b'. The former specifies a "
+                  "column from the first input and the latter a "
+                  "column from the second. The characters after them "
+                  "can be any column identifier (number, name, or "
+                  "regular expression). For more on column selection, "
+                  "please run this command:\n\n"
+                  "    $ info gnuastro \"Selecting table columns\"\n",
+                  col);
+          }
     }
 
-  /* Revere the lists so they correspond to the input order. */
+  /* Reverse the lists so they correspond to the input order. */
   gal_list_str_reverse(&p->acols);
   gal_list_str_reverse(&p->bcols);
 }
@@ -846,6 +997,7 @@ static void
 ui_preparations_out_name(struct matchparams *p)
 {
   /* To temporarily keep the original value. */
+  char *suffix;
   uint8_t keepinputdir_orig;
   char *refname = p->input1name ? p->input1name : p->input2name;
 
@@ -874,14 +1026,19 @@ ui_preparations_out_name(struct matchparams *p)
     }
   else
     {
-      if(p->outcols || p->coord)
+      if(p->outcols || p->coord || p->kdtreemode==MATCH_KDTREE_BUILD)
         {
           if(p->cp.output)
             gal_checkset_allocate_copy(p->cp.output, &p->out1name);
           else
-            p->out1name = gal_checkset_automatic_output(&p->cp,
-                 refname, ( p->cp.tableformat==GAL_TABLE_FORMAT_TXT
-                                  ? "_matched.txt" : "_matched.fits") );
+            {
+              suffix = ( p->kdtreemode==MATCH_KDTREE_BUILD
+                         ? "_kdtree.fits"
+                         : ( p->cp.tableformat==GAL_TABLE_FORMAT_TXT
+                             ? "_matched.txt" : "_matched.fits") );
+              p->out1name = gal_checkset_automatic_output(&p->cp,
+                                                          refname, suffix);
+            }
           gal_checkset_writable_remove(p->out1name, 0, p->cp.dontdelete);
         }
       else
@@ -959,12 +1116,13 @@ ui_preparations(struct matchparams *p)
 
   /* Currently Match only works on catalogs. */
   if(p->mode==MATCH_MODE_WCS)
-    error(EXIT_FAILURE, 0, "currently Match only works on catalogs, we will "
-          "implement the WCS matching routines later");
+    error(EXIT_FAILURE, 0, "currently Match only works on catalogs, "
+          "we will implement the WCS matching routines later");
   else
     {
       ui_read_columns(p);
-      if(p->outcols) ui_preparations_out_cols(p);
+      if(p->outcols)
+        ui_preparations_out_cols(p);
     }
 
   /* Set the output filename. */
@@ -995,6 +1153,7 @@ ui_preparations(struct matchparams *p)
 void
 ui_read_check_inputs_setup(int argc, char *argv[], struct matchparams *p)
 {
+  size_t nthreads;
   struct gal_options_common_params *cp=&p->cp;
 
 
@@ -1047,7 +1206,33 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct matchparams *p)
   /* If the output is a FITS table, prepare all the options as FITS
      keywords to write in output later. */
   if(gal_fits_name_is_fits(p->out1name))
-      gal_options_as_fits_keywords(&p->cp);
+    gal_options_as_fits_keywords(&p->cp);
+
+
+  /* Let the user know that processing has started. */
+  if(!p->cp.quiet)
+    {
+      printf(PROGRAM_NAME" "PACKAGE_VERSION" started on %s",
+             ctime(&p->rawtime));
+      nthreads=p->kdtreemode==MATCH_KDTREE_DISABLE ? 1 : p->cp.numthreads;
+      printf("  - Using %zu CPU thread%s%s\n", nthreads,
+             nthreads==1 ? "." : "s.",
+             ( p->kdtreemode==MATCH_KDTREE_DISABLE
+               ? " (sort-based match only uses a single thread)" : ""));
+      printf("  - Match algorithm: %s\n",
+             p->kdtree ? "k-d tree" : "sort-based");
+      printf("  - Input-1: %s; %zu rows\n",
+             gal_fits_name_save_as_string(p->input1name, p->cp.hdu),
+             p->cols1->size);
+      if(p->kdtreemode==MATCH_KDTREE_FILE)
+        printf("  - Input-1 k-d tree: %s\n",
+               gal_fits_name_save_as_string(p->kdtree, p->kdtreehdu));
+      if(p->kdtreemode!=MATCH_KDTREE_BUILD)
+        printf("  - Input-2: %s; %zu rows\n",
+               p->coord ? "from --coord"
+               : gal_fits_name_save_as_string(p->input2name, p->hdu2),
+               p->cols2->size);
+    }
 }
 
 
@@ -1083,12 +1268,16 @@ ui_free_report(struct matchparams *p, struct timeval *t1)
   free(p->cp.output);
   gal_data_free(p->ccol1);
   gal_data_free(p->ccol2);
+  gal_data_free(p->outcols);
   gal_list_data_free(p->cols1);
   gal_list_data_free(p->cols2);
+  if(p->kdtree) free(p->kdtree);
+  gal_list_str_free(p->acols, 0);
+  gal_list_str_free(p->bcols, 0);
+  if(p->kdtreehdu) free(p->kdtreehdu);
   gal_list_str_free(p->stdinlines, 1);
 
-  /* Print the final message.
+  /* Print the final message. */
   if(!p->cp.quiet)
     gal_timing_report(t1, PROGRAM_NAME" finished in: ", 0);
-  */
 }
