@@ -5,7 +5,7 @@ MakeCatalog is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2021, Free Software Foundation, Inc.
+Copyright (C) 2016-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -105,11 +105,13 @@ enum option_keys_enum
   UI_KEY_CHECKUPLIM,
   UI_KEY_NOCLUMPSORT,
   UI_KEY_FRACMAX,
+  UI_KEY_SPATIALRESOLUTION,
 
   UI_KEY_OBJID,                         /* Catalog columns. */
   UI_KEY_IDINHOSTOBJ,
   UI_KEY_AREAARCSEC2,
   UI_KEY_SURFACEBRIGHTNESS,
+  UI_KEY_SBERROR,
   UI_KEY_AREAXY,
   UI_KEY_CLUMPSAREA,
   UI_KEY_WEIGHTAREA,
@@ -153,11 +155,13 @@ enum option_keys_enum
   UI_KEY_BRIGHTNESSERR,
   UI_KEY_CLUMPSBRIGHTNESS,
   UI_KEY_BRIGHTNESSNORIVER,
+  UI_KEY_STD,
   UI_KEY_MEAN,
   UI_KEY_MEDIAN,
   UI_KEY_MAXIMUM,
   UI_KEY_CLUMPSMAGNITUDE,
   UI_KEY_UPPERLIMIT,
+  UI_KEY_UPPERLIMITSB,
   UI_KEY_UPPERLIMITONESIGMA,
   UI_KEY_UPPERLIMITSIGMA,
   UI_KEY_UPPERLIMITQUANTILE,
@@ -165,7 +169,7 @@ enum option_keys_enum
   UI_KEY_RIVERAVE,
   UI_KEY_RIVERNUM,
   UI_KEY_SKY,
-  UI_KEY_STD,
+  UI_KEY_SKYSTD,
   UI_KEY_SIGCLIPNUMBER,
   UI_KEY_SIGCLIPMEDIAN,
   UI_KEY_SIGCLIPMEAN,
