@@ -5,7 +5,7 @@ MakeNoise is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2021, Free Software Foundation, Inc.
+Copyright (C) 2015-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -32,6 +32,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 #include <gsl/gsl_rng.h>         /* Used in setrandoms.   */
 #include <gnuastro/fits.h>
+#include <gnuastro/units.h>
 #include <gsl/gsl_randist.h>     /* To make noise.        */
 
 #include <gnuastro-internal/timing.h>
@@ -60,7 +61,8 @@ convertsaveoutput(struct mknoiseparams *p)
   gal_fits_list_key_t *headers=NULL;
 
   /* Add the proper information to the header of the output: */
-  gal_fits_key_write_filename("INF", p->inputname, &headers, 0);
+  gal_fits_key_write_filename("INF", p->inputname, &headers, 0,
+                              p->cp.quiet);
   if( !isnan(p->background) )
     {
       gal_checkset_allocate_copy("BCKGRND", &keyname);
@@ -70,7 +72,7 @@ convertsaveoutput(struct mknoiseparams *p)
                                 0, NULL, 0);
       if( !isnan(p->zeropoint) )
         {
-          tmp=-2.5 * log10(p->background) + p->zeropoint;
+          tmp=gal_units_counts_to_mag(p->background, p->zeropoint);
           gal_checkset_allocate_copy("BCKGMAG", &keyname);
           gal_fits_key_list_add_end(&headers, GAL_TYPE_FLOAT64, keyname, 1,
                                     &tmp, 0,
@@ -117,7 +119,8 @@ convertsaveoutput(struct mknoiseparams *p)
   p->input->name=NULL; /* because we didn't allocate it. */
 
   /* Write the configuration keywords. */
-  gal_fits_key_write_filename("input", p->inputname, &p->cp.okeys, 1);
+  gal_fits_key_write_filename("input", p->inputname, &p->cp.okeys, 1,
+                              p->cp.quiet);
   gal_fits_key_write_config(&p->cp.okeys, "MakeNoise configuration",
                             "MKNOISE-CONFIG", p->cp.output, "0");
 }
