@@ -5,7 +5,7 @@ Segment is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2018-2021, Free Software Foundation, Inc.
+Copyright (C) 2018-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -100,7 +100,7 @@ struct argp_option program_options[] =
     {
       "detection",
       UI_KEY_DETECTION,
-      "STR",
+      "FITS",
       0,
       "Filename of detection image (to segment).",
       GAL_OPTIONS_GROUP_INPUT,
@@ -126,7 +126,7 @@ struct argp_option program_options[] =
     {
       "kernel",
       UI_KEY_KERNEL,
-      "STR",
+      "FITS",
       0,
       "Filename of kernel to convolve with input.",
       GAL_OPTIONS_GROUP_INPUT,
@@ -152,7 +152,7 @@ struct argp_option program_options[] =
     {
       "convolved",
       UI_KEY_CONVOLVED,
-      "STR",
+      "FITS",
       0,
       "Convolved image file to avoid convolution.",
       GAL_OPTIONS_GROUP_INPUT,
@@ -186,7 +186,7 @@ struct argp_option program_options[] =
       UI_KEY_RAWOUTPUT,
       0,
       0,
-      "Output only object and clump labels.",
+      "Output contains only CLUMPS and/or OBJECTS HDUs.",
       GAL_OPTIONS_GROUP_OUTPUT,
       &p->rawoutput,
       GAL_OPTIONS_NO_ARG_TYPE,
@@ -195,13 +195,13 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET
     },
     {
-      "onlyclumps",
-      UI_KEY_ONLYCLUMPS,
+      "noobjects",
+      UI_KEY_NOOBJECTS,
       0,
       0,
-      "Finish after finding true clumps.",
+      "Finish with true clumps, don't create objects.",
       GAL_OPTIONS_GROUP_OUTPUT,
-      &p->onlyclumps,
+      &p->noobjects,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
       GAL_OPTIONS_NOT_MANDATORY,
