@@ -5,7 +5,7 @@ Segment is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2020, Free Software Foundation, Inc.
+Copyright (C) 2015-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -66,8 +66,8 @@ clumps_grow_prepare_initial(struct clumps_thread_params *cltprm)
   gal_data_t *input=cltprm->clprm->p->input;
   struct segmentparams *p=cltprm->clprm->p;
 
-  size_t ndiffuse=0, coord[2], *dindexs;
-  double wcoord[2]={0.0f,0.0f}, sum=0.0f;
+  double wcoord[3]={0.0f,0.0f,0.0f}, sum=0.0f;
+  size_t ndiffuse=0, coord[3], tcoord[3], *dindexs;
   size_t *s, *sf, *dsize=input->dsize, ndim=input->ndim;
   float glimit, *imgss=input->array, *std=p->std->array;
   int32_t *olabel=p->olabel->array, *clabel=p->clabel->array;
@@ -79,9 +79,12 @@ clumps_grow_prepare_initial(struct clumps_thread_params *cltprm)
   do
     if( imgss[ *s ] > 0.0f )
       {
-        sum        += imgss[ *s ];
-        wcoord[0]  += imgss[ *s ] * (*s/dsize[1]);
-        wcoord[1]  += imgss[ *s ] * (*s%dsize[1]);
+        gal_dimension_index_to_coord(*s, ndim, dsize, tcoord);
+        sum         += imgss[ *s ];
+        wcoord[0]   += imgss[ *s ] * tcoord[0];
+        wcoord[1]   += imgss[ *s ] * tcoord[1];
+        if(ndim==3)
+          wcoord[1] += imgss[ *s ] * tcoord[2];
       }
   while(++s<sf);
 
@@ -93,8 +96,11 @@ clumps_grow_prepare_initial(struct clumps_thread_params *cltprm)
       sf=(s=indexs->array)+indexs->size;
       do
         {
-          wcoord[0] += *s / dsize[1];
-          wcoord[1] += *s % dsize[1];
+          gal_dimension_index_to_coord(*s, ndim, dsize, tcoord);
+          wcoord[0]   += tcoord[0];
+          wcoord[1]   += tcoord[1];
+          if(ndim==3)
+            wcoord[2] += tcoord[2];
         }
       while(++s<sf);
       sum = indexs->size;
@@ -104,6 +110,8 @@ clumps_grow_prepare_initial(struct clumps_thread_params *cltprm)
   /* Convert floating point coordinates to integers. */
   coord[0] = GAL_DIMENSION_FLT_TO_INT(wcoord[0]/sum);
   coord[1] = GAL_DIMENSION_FLT_TO_INT(wcoord[1]/sum);
+  if(ndim==3)
+    coord[2] = GAL_DIMENSION_FLT_TO_INT(wcoord[2]/sum);
 
 
   /* Find the growth limit. Note that the STD may be a value, or a dataset
@@ -295,14 +303,15 @@ clumps_find_make_sn_table(void *in_prm)
 
   void *tarray;
   double numdet;
+  int pixonedge;
   gal_data_t *tile, *tblock, *tmp;
   uint8_t *binary=p->binary->array;
   struct clumps_thread_params cltprm;
-  size_t i, c, ind, tind, num, numsky, *indarr;
+  size_t i, j, c, ind, tind, num, numsky, *indarr;
   size_t *scoord=gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 0, __func__,
-                                       "scoord");
+                                      "scoord");
   size_t *icoord=gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 0, __func__,
-                                       "icoord");
+                                      "icoord");
 
 
   /* Initialize the parameters for this thread. */
@@ -392,12 +401,19 @@ clumps_find_make_sn_table(void *in_prm)
               ind = (int32_t *)i - (int32_t *)(p->clabel->array);
               gal_dimension_index_to_coord(ind, ndim, dsize, icoord);
 
-              /* If the pixel is on the tile edge, set it as river and
-                 don't include it in the indexs. */
-              if( icoord[0]==scoord[0]
-                  || icoord[0]==scoord[0]+tile->dsize[0]-1
-                  || icoord[1]==scoord[1]
-                  || icoord[1]==scoord[1]+tile->dsize[1]-1 )
+              /* Check if the pixel is on the tile edge. */
+              pixonedge=0;
+              for(j=0;j<ndim;++j)
+                if( icoord[j]==scoord[j]
+                    || icoord[j]==scoord[j]+tile->dsize[j]-1 )
+                  {
+                    pixonedge=1;
+                    break;
+                  }
+
+              /* If this pixel is on the edge, then it should be a
+                 river. */
+              if(pixonedge)
                 *(int32_t *)i=GAL_LABEL_RIVER;
 
               /* This pixel is not on the edge, check if it had a value of
@@ -626,7 +642,8 @@ clumps_true_find_sn_thresh(struct segmentparams *p)
 
           /* Do this step. */
           gal_threads_spin_off(clumps_find_make_sn_table, &clprm,
-                               p->ltl.tottiles, p->cp.numthreads);
+                               p->ltl.tottiles, p->cp.numthreads,
+                               p->cp.minmapsize, p->cp.quietmmap);
 
           /* Set the extension name. */
           switch(clprm.step)
@@ -658,7 +675,8 @@ clumps_true_find_sn_thresh(struct segmentparams *p)
     {
       clprm.step=0;
       gal_threads_spin_off(clumps_find_make_sn_table, &clprm,
-                           p->ltl.tottiles, p->cp.numthreads);
+                           p->ltl.tottiles, p->cp.numthreads,
+                           p->cp.minmapsize, p->cp.quietmmap);
     }
 
 
