@@ -5,7 +5,7 @@ Segment is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2018-2021, Free Software Foundation, Inc.
+Copyright (C) 2018-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -184,16 +184,17 @@ parse_opt(int key, char *arg, struct argp_state *state)
   /* Set the key to this option. */
   switch(key)
     {
-
     /* Read the non-option tokens (arguments): */
     case ARGP_KEY_ARG:
+      /* The user may give a shell variable that is empty! In that case
+         'arg' will be an empty string! We don't want to account for such
+         cases (and give a clear error that no input has been given). */
       if(p->inputname)
         argp_error(state, "only one argument (input file) should be given");
       else
-        p->inputname=arg;
+        if(arg[0]!='\0') p->inputname=arg;
       break;
 
-
     /* This is an option, set its value. */
     default:
       return gal_options_set_from_key(key, arg, p->cp.poptions, &p->cp);
@@ -266,7 +267,7 @@ ui_read_check_only_options(struct segmentparams *p)
       gal_checkset_check_file(p->kernelname);
 
       /* If its FITS, see if a HDU has been provided. */
-      if( gal_fits_name_is_fits(p->kernelname) && p->khdu==NULL )
+      if( gal_fits_file_recognized(p->kernelname) && p->khdu==NULL )
         error(EXIT_FAILURE, 0, "no HDU specified for kernel. When the "
               "kernel is a FITS file, a HDU must also be specified. You "
               "can use the '--khdu' option and give it the HDU number "
@@ -305,7 +306,7 @@ ui_check_options_and_arguments(struct segmentparams *p)
       gal_checkset_check_file(p->inputname);
 
       /* If it is FITS, a HDU is also mandatory. */
-      if( gal_fits_name_is_fits(p->inputname) && p->cp.hdu==NULL )
+      if( gal_fits_file_recognized(p->inputname) && p->cp.hdu==NULL )
         error(EXIT_FAILURE, 0, "no HDU specified. When the input is a FITS "
               "file, a HDU must also be specified, you can use the '--hdu' "
               "('-h') option and give it the HDU number (starting from "
@@ -411,8 +412,10 @@ ui_prepare_inputs(struct segmentparams *p)
   /* Read the input as a single precision floating point dataset. */
   p->input = gal_array_read_one_ch_to_type(p->inputname, p->cp.hdu,
                                            NULL, GAL_TYPE_FLOAT32,
-                                           p->cp.minmapsize, p->cp.quietmmap);
-  p->input->wcs = gal_wcs_read(p->inputname, p->cp.hdu, 0, 0,
+                                           p->cp.minmapsize,
+                                           p->cp.quietmmap);
+  p->input->wcs = gal_wcs_read(p->inputname, p->cp.hdu,
+                               p->cp.wcslinearmatrix, 0, 0,
                                &p->input->nwcs);
   p->input->ndim=gal_dimension_remove_extra(p->input->ndim,
                                             p->input->dsize,
@@ -428,10 +431,10 @@ ui_prepare_inputs(struct segmentparams *p)
 
 
   /* Segment currently only works on 2D datasets (images). */
-  if(p->input->ndim!=2)
+  if(p->input->ndim!=2 && p->input->ndim!=3)
     error(EXIT_FAILURE, 0, "%s (hdu: %s) has %zu dimensions but Segment "
-          "can only operate on 2D datasets (images)", p->inputname, p->cp.hdu,
-          p->input->ndim);
+          "can only operate on 2D (images) or 3D (cube) datasets",
+          p->inputname, p->cp.hdu, p->input->ndim);
 
 
   /* If a convolved image is given, read it. */
@@ -537,27 +540,39 @@ ui_prepare_inputs(struct segmentparams *p)
 
 
 
-/* Prepare the input kernel. */
+/* Prepare the kernel, either from a file, or from the default arrays
+   available in the headers. The default kernels were created as
+   follows. */
 static void
 ui_prepare_kernel(struct segmentparams *p)
 {
   float *f, *ff, *k;
+  size_t ndim=p->input->ndim;
 
 /* Import the default kernel. */
 #include "kernel-2d.h"
+#include "kernel-3d.h"
 
   /* If a kernel file is given, then use it. Otherwise, use the default
      kernel. */
   if(p->kernelname)
     {
+      /* Read the kernel. */
       if( strcmp(p->kernelname, UI_NO_CONV_KERNEL_NAME) )
         {
+          /* Read the kernel into memory. */
           p->kernel=gal_fits_img_read_kernel(p->kernelname, p->khdu,
                                              p->cp.minmapsize,
                                              p->cp.quietmmap);
           p->kernel->ndim=gal_dimension_remove_extra(p->kernel->ndim,
                                                      p->kernel->dsize,
                                                      NULL);
+
+          /* Make sure it has the same dimensions as the input. */
+          if( p->kernel->ndim != p->input->ndim )
+            error(EXIT_FAILURE, 0, "%s (hdu %s): is %zuD, however, %s (%s) "
+                  "is a %zuD dataset", p->kernelname, p->khdu,
+                  p->kernel->ndim, p->inputname, p->cp.hdu, p->input->ndim);
         }
       else
         p->kernel=NULL;
@@ -566,14 +581,15 @@ ui_prepare_kernel(struct segmentparams *p)
     {
       /* Allocate space for the kernel (we don't want to use the statically
          allocated array. */
-      p->kernel=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 2,
-                               kernel_2d_dsize, NULL, 0, p->cp.minmapsize,
-                               p->cp.quietmmap, NULL, NULL, NULL);
+      p->kernel=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, p->input->ndim,
+                               ndim==2 ? kernel_2d_dsize : kernel_3d_dsize,
+                               NULL, 0, p->cp.minmapsize, p->cp.quietmmap,
+                               NULL, NULL, NULL);
 
-      /* Now copy the staticly allocated array into it. */
-      k=p->kernel->array;
-      ff=(f=kernel_2d)+gal_dimension_total_size(2, p->kernel->dsize);
-      do *k++=*f; while(++f<ff);
+      /* Copy the staticly allocated default array into `p->kernel'. */
+      k = ndim==2 ? kernel_2d : kernel_3d;
+      ff = (f=p->kernel->array) + p->kernel->size;
+      do *f=*k++; while(++f<ff);
     }
 }
 
