@@ -5,7 +5,7 @@ Statistics is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2019-2021, Free Software Foundation, Inc.
+Copyright (C) 2019-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -43,7 +43,7 @@ contour_pixels(gal_data_t *input, double level, size_t minmapsize,
 {
   size_t one=1;
   uint8_t *b, *a, *af;
-  int flags=GAL_ARITHMETIC_NUMOK;
+  int flags=GAL_ARITHMETIC_FLAG_NUMOK;
   gal_data_t *number, *thresh, *eroded;
 
   /* Allocate the single-element dataset to use in arithmetic.*/
