@@ -332,6 +332,33 @@ ui_read_check_only_options(struct tableparams *p)
         error(EXIT_FAILURE, 0, "at most three values can be given to each "
               "call of '--colmetadata' ('-m') after the original columns "
               "name or number. But %zu strings have been given", tmp->size);
+
+  /* If '--catcolumns' is given (to only concatenate certain columns), but
+     no file has been given for appending from (to '--catcolumnfile'), then
+     print a warning to let the user know (this may have been a typo!). */
+  if(p->cp.quiet==0 && p->catcolumns && p->catcolumnfile==NULL)
+    error(EXIT_SUCCESS, 0, "WARNING: '--catcolumns' (or '-C') is ignored "
+          "because '--catcolumnfile' (or '-L') was not given to specify "
+          "which file the new columns should come from. To suppress this "
+          "warning, please run with '--quiet'");
+
+  /* Check the floating point format string. */
+  if(p->txtf32fmtstr)
+    {
+      p->txtf32format=gal_table_displayflt_from_str(p->txtf32fmtstr);
+      if(p->txtf32format==GAL_TABLE_DISPLAY_FMT_INVALID)
+        error(EXIT_FAILURE, 0, "'%s' is not a recognized value for "
+              "'--txtf64format'. Recognized values are 'fixed' and "
+              "'exp'", p->txtf32fmtstr);
+    }
+  if(p->txtf64fmtstr)
+    {
+      p->txtf64format=gal_table_displayflt_from_str(p->txtf64fmtstr);
+      if(p->txtf64format==GAL_TABLE_DISPLAY_FMT_INVALID)
+        error(EXIT_FAILURE, 0, "'%s' is not a recognized value for "
+              "'--txtf64format'. Recognized values are 'fixed' and "
+              "'exp'", p->txtf64fmtstr);
+    }
 }
 
 
