@@ -41,8 +41,8 @@ enum program_args_groups
 
 /* Available letters for short options:
 
-   a d f g j k l p t v x y z
-   A B G J O Q Y
+   a g j k l t v x y z
+   A G J Q Y
 */
 enum option_keys_enum
 {
@@ -67,6 +67,10 @@ enum option_keys_enum
   UI_KEY_CATROWFILE      = 'R',
   UI_KEY_CATROWHDU       = 'X',
   UI_KEY_COLMETADATA     = 'm',
+  UI_KEY_TXTF32FORMAT    = 'f',
+  UI_KEY_TXTF64FORMAT    = 'd',
+  UI_KEY_TXTF32PRECISION = 'p',
+  UI_KEY_TXTF64PRECISION = 'B',
 
   /* Only with long version (start with a value 1000, the rest will be set
      automatically). */
