@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2020, Free Software Foundation, Inc.
+Copyright (C) 2017-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -549,7 +549,8 @@ gal_convolve_spatial_general(gal_data_t *tiles, gal_data_t *kernel,
 
   /* Do the spatial convolution on threads. */
   gal_threads_spin_off(convolve_spatial_on_thread, &params,
-                       gal_list_data_number(tiles), numthreads);
+                       gal_list_data_number(tiles), numthreads,
+                       tiles->minmapsize, tiles->quietmmap);
 
 
   /* Clean up and return the output array. */
