@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2021, Free Software Foundation, Inc.
+Copyright (C) 2015-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -121,6 +121,9 @@ gal_fits_name_is_fits(char *name);
 int
 gal_fits_suffix_is_fits(char *suffix);
 
+int
+gal_fits_file_recognized(char *filename);
+
 char *
 gal_fits_name_save_as_string(char *filename, char *hdu);
 
@@ -169,7 +172,8 @@ int
 gal_fits_hdu_is_healpix(fitsfile *fptr);
 
 fitsfile *
-gal_fits_hdu_open(char *filename, char *hdu, int iomode);
+gal_fits_hdu_open(char *filename, char *hdu, int iomode,
+                  int exitonerror);
 
 fitsfile *
 gal_fits_hdu_open_format(char *filename, char *hdu, int img0_tab1);
@@ -180,6 +184,9 @@ gal_fits_hdu_open_format(char *filename, char *hdu, int img0_tab1);
 /**************************************************************/
 /**********            Header keywords             ************/
 /**************************************************************/
+int
+gal_fits_key_exists_fptr(fitsfile *fptr, char *keyname);
+
 void *
 gal_fits_key_img_blank(uint8_t type);
 
@@ -235,10 +242,12 @@ gal_fits_key_write_title_in_ptr(char *title, fitsfile *fptr);
 
 void
 gal_fits_key_write_filename(char *keynamebase, char *filename,
-                            gal_fits_list_key_t **list, int top1end0);
+                            gal_fits_list_key_t **list, int top1end0,
+                            int quiet);
 
 void
-gal_fits_key_write_wcsstr(fitsfile *fptr, char *wcsstr, int nkeyrec);
+gal_fits_key_write_wcsstr(fitsfile *fptr, struct wcsprm *wcs,
+                          char *wcsstr, int nkeyrec);
 
 void
 gal_fits_key_write(gal_fits_list_key_t **keylist, char *title,
@@ -259,6 +268,13 @@ void
 gal_fits_key_write_config(gal_fits_list_key_t **keylist, char *title,
                           char *extname, char *filename, char *hdu);
 
+gal_list_str_t *
+gal_fits_with_keyvalue(gal_list_str_t *files, char *hdu, char *name,
+                       gal_list_str_t *values);
+
+gal_list_str_t *
+gal_fits_unique_keyvalues(gal_list_str_t *files, char *hdu, char *name);
+
 
 
 
@@ -321,8 +337,8 @@ gal_fits_tab_info(char *filename, char *hdu, size_t *numcols,
 
 gal_data_t *
 gal_fits_tab_read(char *filename, char *hdu, size_t numrows,
-                  gal_data_t *colinfo, gal_list_sizet_t *indexll,
-                  size_t minmapsize, int quietmmap);
+                  gal_data_t *allcols, gal_list_sizet_t *indexll,
+                  size_t numthreads, size_t minmapsize, int quietmmap);
 
 void
 gal_fits_tab_write(gal_data_t *cols, gal_list_str_t *comments,
