@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2020, Free Software Foundation, Inc.
+Copyright (C) 2015-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -72,4 +72,4 @@ gal_jpeg_write(gal_data_t *in, char *filename, uint8_t quality,
 
 __END_C_DECLS    /* From C++ preparations */
 
-#endif           /* __GAL_TIFF_H__ */
+#endif           /* __GAL_JPEG_H__ */
