@@ -6,7 +6,7 @@ Original author:
      Kartik Ohri <kartikohri13@gmail.com>
 Contributing author(s):
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
-Copyright (C) 2020-2021, Free Software Foundation, Inc.
+Copyright (C) 2020-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -79,9 +79,32 @@ gal_units_counts_to_mag(double counts, double zeropoint);
 double
 gal_units_mag_to_counts(double mag, double zeropoint);
 
+double
+gal_units_mag_to_sb(double mag, double area_arcsec2);
+
+double
+gal_units_sb_to_mag(double sb, double area_arcsec2);
+
+double
+gal_units_counts_to_sb(double counts, double zeropoint,
+                       double area_arcsec2);
+
+double
+gal_units_sb_to_counts(double sb, double zeropoint,
+                       double area_arcsec2);
+
 double
 gal_units_counts_to_jy(double counts, double zeropoint_ab);
 
+double
+gal_units_jy_to_counts(double jy, double zeropoint_ab);
+
+double
+gal_units_jy_to_mag(double jy);
+
+double
+gal_units_mag_to_jy(double mag);
+
 double
 gal_units_au_to_pc(double au);
 
