@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2021, Free Software Foundation, Inc.
+Copyright (C) 2015-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -31,6 +31,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 #include <gnuastro/eps.h>
 #include <gnuastro/pdf.h>
+#include <gnuastro/jpeg.h>
 
 #include <gnuastro-internal/checkset.h>
 
@@ -99,30 +100,40 @@ gal_pdf_suffix_is_pdf(char *name)
  *************************************************************/
 void
 gal_pdf_write(gal_data_t *in, char *filename, float widthincm,
-              uint32_t borderwidth, int dontoptimize)
+              uint32_t borderwidth, uint8_t bordercolor,
+              int dontoptimize, gal_data_t *marks)
 {
-  char command[20000];
   size_t w_h_in_pt[2];
+  char *command, *device;
   char *epsname=gal_checkset_malloc_cat(filename, ".ps");
 
   /* Write the EPS file. */
-  gal_eps_write(in, epsname, widthincm, borderwidth, 0, dontoptimize, 1);
+  gal_eps_write(in, epsname, widthincm, borderwidth,
+                bordercolor, 0, dontoptimize, 0, marks);
 
   /* Get the size of the image in 'pt' units. */
   gal_eps_to_pt(widthincm, in->dsize, w_h_in_pt);
 
+  /* Set the device from the file name */
+  if(gal_jpeg_name_is_jpeg(filename)) device="jpeg";
+  else                                device="pdfwrite";
+
   /* Write the ghostscript command to compile the EPS file to PDF. */
-  sprintf(command, "gs -q -o %s -sDEVICE=pdfwrite -dDEVICEWIDTHPOINTS=%zu"
-          " -dDEVICEHEIGHTPOINTS=%zu -dPDFFitPage %s", filename,
-          w_h_in_pt[0]+2*borderwidth, w_h_in_pt[1]+2*borderwidth,
-          epsname);
+  if( asprintf(&command, "gs -q -o %s -sDEVICE=%s "
+               "-dDEVICEWIDTHPOINTS=%zu -dDEVICEHEIGHTPOINTS=%zu "
+               "-dPDFFitPage %s", filename, device,
+               w_h_in_pt[0]+2*borderwidth, w_h_in_pt[1]+2*borderwidth,
+               epsname)<0 )
+    error(EXIT_FAILURE, 0, "%s: asprintf allocation error", __func__);
 
   /* Run Ghostscript. */
   if(system(command))
-    error(EXIT_FAILURE, 0, "the command to convert a PostScript file to "
-          "PDF ('%s') was not successful! The PostScript file (%s) is "
-          "left if you want to convert or use it through any other "
-          "means", command, epsname);
+    error(EXIT_FAILURE, 0, "the Ghostscript command (printed after "
+          "this message) to convert the EPS file to PDF was not "
+          "successful! The EPS file ('%s') is left if you want to "
+          "convert it through any other means (for example the "
+          "'epspdf' program). The Ghostscript command was: %s",
+          epsname, command);
 
   /* Delete the EPS file. */
   errno=0;
@@ -130,5 +141,6 @@ gal_pdf_write(gal_data_t *in, char *filename, float widthincm,
     error(EXIT_FAILURE, errno, "%s", epsname);
 
   /* Clean up. */
+  free(command);
   free(epsname);
 }
