@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2018-2021, Free Software Foundation, Inc.
+Copyright (C) 2018-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -46,8 +46,23 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 
 /*************************************************************
- **************      Acceptable TIFF names      **************
+ ************** TIFF name and file identification ************
  *************************************************************/
+#ifndef HAVE_LIBTIFF
+static void
+tiff_error_no_litiff(const char *func)
+{
+  error(EXIT_FAILURE, 0, "%s: libtiff was not found during the "
+        "configuration of %s on this system. To read from TIFF files, "
+        "libtiff is required. Please install libtiff, then configure, make "
+        "and install %s again", func, PACKAGE_STRING, PACKAGE_STRING);
+}
+#endif
+
+
+
+
+
 int
 gal_tiff_name_is_tiff(char *name)
 {
@@ -56,7 +71,7 @@ gal_tiff_name_is_tiff(char *name)
   if(name)
     {
       len=strlen(name);
-      if ( ( len>=3 && strcmp(&name[len-3], "tif") == 0 )
+      if (    ( len>=3 && strcmp(&name[len-3], "tif") == 0 )
            || ( len>=3 && strcmp(&name[len-3], "TIF") == 0 )
            || ( len>=4 && strcmp(&name[len-4], "tiff") == 0 )
            || ( len>=4 && strcmp(&name[len-4], "TIFF") == 0 ) )
@@ -277,7 +292,7 @@ tiff_read_contig_strip_data(TIFF *tif, char *filename, size_t dir,
   unsigned char *buf;
   uint32_t row, rowsperstrip = (uint32_t)-1;
   size_t nrow=0, scanline=TIFFScanlineSize(tif);
-  uint32 h=out->ndim==2?out->dsize[0]:out->dsize[1];
+  uint32_t h=out->ndim==2?out->dsize[0]:out->dsize[1];
 
   /* Allocate the buffer. */
   errno=0;
@@ -327,7 +342,7 @@ tiff_read_separate_strip_data(TIFF* tif, char *filename, size_t dir,
   uint32_t rowsperstrip=(uint32_t)-1;
   size_t nrow=0, scanline=TIFFScanlineSize(tif);
   size_t ostart=0, numch=gal_list_data_number(out);
-  uint32 row, h=out->ndim==2?out->dsize[0]:out->dsize[1];
+  uint32_t row, h=out->ndim==2?out->dsize[0]:out->dsize[1];
 
   /* Allocate the buffer. */
   errno=0;
@@ -608,10 +623,7 @@ gal_tiff_read(char *filename, size_t dir, size_t minmapsize, int quietmmap)
   TIFFClose(tif);
   return out;
 #else
-  error(EXIT_FAILURE, 0, "%s: libtiff was not found during the "
-        "configuration of %s on this system. To read from TIFF files, "
-        "libtiff is required. Please install libtiff, then configure, make "
-        "and install %s again", __func__, PACKAGE_STRING, PACKAGE_STRING);
+  tiff_error_no_litiff(__func__);
   return NULL;
 #endif  /* HAVE_LIBTIFF */
 }
