@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2020, Free Software Foundation, Inc.
+Copyright (C) 2017-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -562,7 +562,8 @@ gal_tile_block_blank_flag(gal_data_t *tile_ll, size_t numthreads)
 {
   /* Go over all the tiles and update their blank flag. */
   gal_threads_spin_off(tile_block_blank_flag, tile_ll,
-                       gal_list_data_number(tile_ll), numthreads);
+                       gal_list_data_number(tile_ll), numthreads,
+                       tile_ll->minmapsize, tile_ll->quietmmap);
 }
 
 
