@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2021, Free Software Foundation, Inc.
+Copyright (C) 2016-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -560,6 +560,7 @@ gal_type_from_string(void **out, char *string, uint8_t type)
 void *
 gal_type_string_to_number(char *string, uint8_t *type)
 {
+  long int l;
   void *ptr, *out;
   int fnz=-1, lnz=0;     /* 'F'irst (or 'L'ast) 'N'on-'Z'ero. */
   uint8_t forcedfloat=0;
@@ -581,18 +582,48 @@ gal_type_string_to_number(char *string, uint8_t *type)
      force a float and this loop is necessary in such cases. */
   for(c=string; *c!='\0'; ++c) if(*c=='.') { forcedfloat=1; break; }
 
-  /* See if the number is actually an integer: */
-  if( forcedfloat==0 && ceil(d) == d )
+  /* Read the number as an integer if 1) we aren't in forced-float mode, 2)
+     the number is actually an integer ('ceil(d)==d'), and 3) the number
+     fits within interger limits: the maximum value of an unsigned 64-bit
+     integer is almost 1.8e19, and the minimum value of a signed 64-bit
+     integer is almost -9.2e18), see the "Numeric data types" section of
+     Gnuastro's book. */
+  if( forcedfloat==0 && ceil(d) == d && d<1.8e19f && d>-9.2e18f )
     {
+      /* We know the number is an integer, so we should re-read it again,
+         but this time, as an integer, because: 1) floating point numbers
+         can only preserve a certain number of decimals precisely after a
+         certain number of decimals, they loose precision. 2) Integer
+         comparisons (that are done below) are faster, but this is
+         secondary because the parsing itself takes more time!
+
+         The string is being parsed as an integer in base-10 (third
+         argument of 'strtol'). This should not be '0', otherwise 'strtol'
+         will parse strings starting with '0' in octal radix and strings
+         starting in '0x' in hexagesimal radix (these aren't used in data
+         analysis, only in computer science). */
+      l=strtol(string, &tailptr, 10);
+      if(*tailptr!='\0')
+        {
+          /* If 'tailptr' is simply an 'e', the input string was in
+             scientific notation (for example '1e5'). In such cases, the
+             number of decimals is (usually!) enough to fit in a double,
+             and we can simply put the value of 'd' in 'l'. Note that with
+             'ceil(d)==d' we have confirmed that the number is actually an
+             integer (and not a float). */
+          if(*tailptr=='e') l=d;
+          else return NULL;
+        }
+
       /* If the number is negative, put it in the signed types (based on
          its value). If its zero or positive, then put it in the unsigned
          types. */
-      if( d < 0 )
+      if( l < 0 )
         {
-          if     (d>INT8_MIN)    { i8=d;  ptr=&i8;  *type=GAL_TYPE_INT8;   }
-          else if(d>INT16_MIN)   { i16=d; ptr=&i16; *type=GAL_TYPE_INT16;  }
-          else if(d>INT32_MIN)   { i32=d; ptr=&i32; *type=GAL_TYPE_INT32;  }
-          else                   { i64=d; ptr=&i64; *type=GAL_TYPE_INT64;  }
+          if     (l>INT8_MIN)    { i8=l;  ptr=&i8;  *type=GAL_TYPE_INT8;  }
+          else if(l>INT16_MIN)   { i16=l; ptr=&i16; *type=GAL_TYPE_INT16; }
+          else if(l>INT32_MIN)   { i32=l; ptr=&i32; *type=GAL_TYPE_INT32; }
+          else                   { i64=l; ptr=&i64; *type=GAL_TYPE_INT64; }
         }
       else
         {
@@ -602,11 +633,25 @@ gal_type_string_to_number(char *string, uint8_t *type)
              confusing situations (for example when the user gives 255), if
              the value is equal to the given maximum of the given type,
              we'll assign it to a larger type. In other words, we won't be
-             using the '<=MAX', but '<MAX'. */
-          if     (d<UINT8_MAX)  { u8=d;  ptr=&u8;  *type=GAL_TYPE_UINT8;  }
-          else if(d<UINT16_MAX) { u16=d; ptr=&u16; *type=GAL_TYPE_UINT16; }
-          else if(d<UINT32_MAX) { u32=d; ptr=&u32; *type=GAL_TYPE_UINT32; }
-          else                  { u64=d; ptr=&u64; *type=GAL_TYPE_UINT64; }
+             using the '<=MAX', but '<MAX'.
+
+             Even though they are positive, we should give priority to the
+             signed types if the number fits in the range of signed type
+             for that width: this is the way that C's internal automatic
+             type conversion works (which is used by Arithmetic's binary
+             operators for example). */
+          if     (l<UINT8_MAX)
+            {  if(l>INT8_MAX)  { u8=l;  ptr=&u8;  *type=GAL_TYPE_UINT8; }
+               else            { i8=l;  ptr=&i8;  *type=GAL_TYPE_INT8;  } }
+          else if(l<UINT16_MAX)
+            {  if(l>INT16_MAX) { u16=l; ptr=&u16; *type=GAL_TYPE_UINT16; }
+               else            { i16=l; ptr=&i16; *type=GAL_TYPE_INT16;  } }
+          else if(l<UINT32_MAX)
+            {  if(l>INT32_MAX) { u32=l; ptr=&u32; *type=GAL_TYPE_UINT32; }
+               else            { i32=l; ptr=&i32; *type=GAL_TYPE_INT32;  } }
+          else
+            {  if(l>INT64_MAX) { u64=l; ptr=&u64; *type=GAL_TYPE_UINT64; }
+               else            { i64=l; ptr=&i64; *type=GAL_TYPE_INT64;  } }
         }
     }
   else
@@ -639,10 +684,10 @@ gal_type_string_to_number(char *string, uint8_t *type)
 
       /* Calculate the number of decimal digits and decide if it the number
          should be a float or a double. */
-      if( lnz-fnz < FLT_DIG || ( d<FLT_MAX && d>FLT_MIN ) )
-        { f=d; ptr=&f; *type=GAL_TYPE_FLOAT32; }
-      else
+      if( lnz-fnz > FLT_DIG || fabs(d)>FLT_MAX || fabs(d)<FLT_MIN )
         {      ptr=&d; *type=GAL_TYPE_FLOAT64; }
+      else
+        { f=d; ptr=&f; *type=GAL_TYPE_FLOAT32; }
     }
 
   /* Allocate a one-element dataset, then copy the number into it. */
