@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2021, Free Software Foundation, Inc.
+Copyright (C) 2015-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -33,6 +33,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 #include <gsl/gsl_linalg.h>
 #include <wcslib/wcsmath.h>
+#include <wcslib/wcsprintf.h>
 
 #include <gnuastro/wcs.h>
 #include <gnuastro/tile.h>
@@ -123,6 +124,24 @@ wcs_read_correct_pc_cd(struct wcsprm *wcs)
 
 
 
+/* For the TPV, TNX and ZPX distortions, WCSLIB can't deal with the CDELT
+   keys properly and its better to use the CD matrix instead, this function
+   will check for this and return 1 if a CD matrix should be used
+   (over-riding the user's desired matrix if necessary). */
+static int
+wcs_use_cd_for_distortion(struct wcsprm *wcs)
+{
+  return ( wcs
+           && wcs->lin.disseq
+           && ( !strcmp(   wcs->lin.disseq->dtype[1], "TPV")
+                || !strcmp(wcs->lin.disseq->dtype[1], "TNX")
+                || !strcmp(wcs->lin.disseq->dtype[1], "ZPX") ) );
+}
+
+
+
+
+
 /* Read the WCS information from the header. Unfortunately, WCS lib is
    not thread safe, so it needs a mutex. In case you are not using
    multiple threads, just pass a NULL pointer as the mutex.
@@ -201,11 +220,8 @@ gal_wcs_read_fitsptr(fitsfile *fptr, int linearmatrix, size_t hstartwcs,
   status=wcspih(fullheader, nkeys, relax, ctrl, &nreject, nwcs, &wcs);
   if(status)
     {
-      fprintf(stderr,"\n"
-              "##################\n"
-              "WCSLIB Warning: wcspih ERROR %d: %s.\n"
-              "##################\n",
-              status, wcs_errmsg[status]);
+      error(EXIT_SUCCESS, 0, "%s: WCSLIB Warning: wcspih ERROR %d: %s",
+            __func__, status, wcs_errmsg[status]);
       wcs=NULL; *nwcs=0;
     }
 
@@ -326,14 +342,21 @@ gal_wcs_read_fitsptr(fitsfile *fptr, int linearmatrix, size_t hstartwcs,
                       __func__, fixstatus[CYLFIX]);
             }
 
+          /* Enable wcserr if necessary (can help in debugging situations
+             when the default error message isn't enough).  If you confront
+             an error, un-comment the two lines below and put
+             'wcsperr(wcs,NULL);' after the problematic WCSLIB
+             command. This will print a trace-back of the cause.
+          wcserr_enable(1);
+          wcsprintf_set(stderr);
+          */
+
           /* Set the WCS structure. */
           status=wcsset(wcs);
           if(status)
             {
-              fprintf(stderr, "\n##################\n"
-                      "WCSLIB Warning: wcsset ERROR %d: %s.\n"
-                      "##################\n",
-                      status, wcs_errmsg[status]);
+              error(EXIT_SUCCESS, 0, "%s: WCSLIB warning: wcsset "
+                    "error %d: %s", __func__, status, wcs_errmsg[status]);
               wcsfree(wcs);
               wcs=NULL;
               *nwcs=0;
@@ -359,10 +382,12 @@ gal_wcs_read_fitsptr(fitsfile *fptr, int linearmatrix, size_t hstartwcs,
         }
     }
 
-  /* If the user wants a CD linear matrix, do the conversion here,
-     otherwise, make sure the PC matrix is used. */
-  if(linearmatrix==GAL_WCS_LINEAR_MATRIX_CD) gal_wcs_to_cd(wcs);
-  else                          gal_wcs_decompose_pc_cdelt(wcs);
+  /* If the distortion requires a CD matrix, or if the user wants it, do
+     the conversion here, otherwise, make sure the PC matrix is used. */
+  if(wcs_use_cd_for_distortion(wcs) \
+     || linearmatrix==GAL_WCS_LINEAR_MATRIX_CD)
+    gal_wcs_to_cd(wcs);
+  else gal_wcs_decompose_pc_cdelt(wcs);
 
   /* Clean up and return. */
   status=0;
@@ -443,8 +468,8 @@ gal_wcs_create(double *crpix, double *crval, double *cdelt,
   /* Set up the wcs structure with the constants defined above. */
   status=wcsset(wcs);
   if(status)
-    error(EXIT_FAILURE, 0, "wcsset error %d: %s", status,
-          wcs_errmsg[status]);
+    error(EXIT_FAILURE, 0, "%s: wcsset error %d: %s", __func__,
+          status, wcs_errmsg[status]);
 
   /* If a CD matrix is desired make it. */
   if(linearmatrix==GAL_WCS_LINEAR_MATRIX_CD)
@@ -458,6 +483,28 @@ gal_wcs_create(double *crpix, double *crval, double *cdelt,
 
 
 
+void
+gal_wcs_free(struct wcsprm *wcs)
+{
+  int status;
+
+  /* If it is already NULL, then don't continue. */
+  if(wcs==NULL) return;
+
+  /* Free the internal structure. */
+  status=wcsfree(wcs);
+  if(status)
+    error(EXIT_FAILURE, 0, "%s: WCSLIB wcsfree ERROR %d: %s",
+          __func__, status, wcs_errmsg[status]);
+
+  /* Free the actual structure. */
+  free(wcs);
+}
+
+
+
+
+
 /* Extract the dimension name from CTYPE. */
 char *
 gal_wcs_dimension_name(struct wcsprm *wcs, size_t dimension)
@@ -502,70 +549,53 @@ gal_wcs_dimension_name(struct wcsprm *wcs, size_t dimension)
 /*************************************************************
  ***********               Write WCS               ***********
  *************************************************************/
-void
-gal_wcs_write_in_fitsptr(fitsfile *fptr, struct wcsprm *wcs)
+char *
+gal_wcs_write_wcsstr(struct wcsprm *wcs, int *nkeyrec)
 {
   char *wcsstr;
-  int cdfordist, status=0, nkeyrec;
-
-  /* For the TPV, TNX and ZPX distortions, WCSLIB can't deal with the CDELT
-     keys properly and its better to use the CD matrix instead, so we'll
-     use the 'gal_wcs_to_cd' function. */
-  cdfordist = ( wcs->lin.disseq
-                && ( !strcmp(   wcs->lin.disseq->dtype[1], "TPV")
-                     || !strcmp(wcs->lin.disseq->dtype[1], "TNX")
-                     || !strcmp(wcs->lin.disseq->dtype[1], "ZPX") ) );
+  int status=0;
 
   /* Finalize the linear transformation matrix. Note that some programs may
      have worked on the WCS. So even if 'altlin' is already 2, we'll just
      ensure that the final matrix is CD here. */
-  if(wcs->altlin==2 || cdfordist) gal_wcs_to_cd(wcs);
-  else                            gal_wcs_decompose_pc_cdelt(wcs);
+  if(wcs->altlin==2 || wcs_use_cd_for_distortion(wcs)) gal_wcs_to_cd(wcs);
+  else gal_wcs_decompose_pc_cdelt(wcs);
 
   /* Clean up small errors in the PC matrix and CDELT values. */
   gal_wcs_clean_errors(wcs);
 
-  /* Convert the WCS information to text. */
-  status=wcshdo(WCSHDO_safe, wcs, &nkeyrec, &wcsstr);
+  /* Convert the WCS information to text. If there was an error, then free
+     any allocated space and put zero on 'nkeyrec'. */
+  status=wcshdo(WCSHDO_safe, wcs, nkeyrec, &wcsstr);
   if(status)
-    error(0, 0, "%s: WARNING: WCSLIB error, no WCS in output.\n"
-          "wcshdu ERROR %d: %s", __func__, status, wcs_errmsg[status]);
-  else
     {
-      gal_fits_key_write_wcsstr(fptr, wcs, wcsstr, nkeyrec);
-      free(wcsstr);
+      error(0, 0, "%s: WARNING: WCSLIB error, no WCS in output.\n"
+            "wcshdo ERROR %d: %s", __func__, status, wcs_errmsg[status]);
+      if(wcsstr) free(wcsstr);
+      *nkeyrec=0;
+      wcsstr=NULL;
     }
-  status=0;
 
-   /* WCSLIB is going to write PC+CDELT keywords in any case. But when we
-      have a TPV, TNX or ZPX distortion, it is cleaner to use a CD matrix
-      (WCSLIB can't convert coordinates properly if the PC matrix is used
-      with the TPV distortion). So to help users avoid the potential
-      problems with WCSLIB. 'gal_wcs_to_cd' function already made sure that
-      CDELTi=1.0, so effectively the CD matrix and PC matrix are
-      equivalent, we just need to convert the keyword names and delete the
-      CDELT keywords. Note that zero-valued PC/CD elements may not be
-      present, so we'll manually set 'status' to zero to avoid CFITSIO from
-      crashing. */
-  if(wcs->altlin==2)
-    {
-      status=0; fits_delete_str(fptr, "CDELT1", &status);
-      status=0; fits_delete_str(fptr, "CDELT2", &status);
-      status=0; fits_modify_name(fptr, "PC1_1", "CD1_1", &status);
-      status=0; fits_modify_name(fptr, "PC1_2", "CD1_2", &status);
-      status=0; fits_modify_name(fptr, "PC2_1", "CD2_1", &status);
-      status=0; fits_modify_name(fptr, "PC2_2", "CD2_2", &status);
-      if(wcs->naxis==3)
-        {
-          status=0; fits_delete_str(fptr, "CDELT3", &status);
-          status=0; fits_modify_name(fptr, "PC1_3", "CD1_3", &status);
-          status=0; fits_modify_name(fptr, "PC2_3", "CD2_3", &status);
-          status=0; fits_modify_name(fptr, "PC3_1", "CD3_1", &status);
-          status=0; fits_modify_name(fptr, "PC3_2", "CD3_2", &status);
-          status=0; fits_modify_name(fptr, "PC3_3", "CD3_3", &status);
-        }
-      status=0;
-    }
+  /* Return the string. */
+  return wcsstr;
+}
+
+
+
+
+
+void
+gal_wcs_write_in_fitsptr(fitsfile *fptr, struct wcsprm *wcs)
+{
+  char *wcsstr;
+  int nkeyrec=0;
+
+  /* Convert the WCS structure into FITS keywords as a string. */
+  wcsstr=gal_wcs_write_wcsstr(wcs, &nkeyrec);
+
+  /* Write the keywords into the FITS file. */
+  gal_fits_key_write_wcsstr(fptr, wcs, wcsstr, nkeyrec);
+  free(wcsstr);
 }
 
 
@@ -1136,23 +1166,25 @@ gal_wcs_distortion_identify(struct wcsprm *wcs)
 
   if( dispre != NULL )
     {
-      if(      !strcmp(*dispre->dtype, "SIP") ) return GAL_WCS_DISTORTION_SIP;
-      else if( !strcmp(*dispre->dtype, "TPD") ) return GAL_WCS_DISTORTION_TPD;
+      if(      !strcmp(*dispre->dtype, "SIP") )
+        return GAL_WCS_DISTORTION_SIP;
+      else if( !strcmp(*dispre->dtype, "TPD") )
+        return GAL_WCS_DISTORTION_TPD;
       else
-        error(EXIT_FAILURE, 0, "%s: distortion '%s' isn't recognized in "
-              "the 'dispre' structure of the given 'wcsprm'", __func__,
-              *dispre->dtype);
+        return GAL_WCS_DISTORTION_INVALID;
     }
   else if( disseq != NULL )
     {
-      if(      !strcmp(*disseq->dtype, "TPV") ) return GAL_WCS_DISTORTION_TPV;
-      else if( !strcmp(*disseq->dtype, "TPD") ) return GAL_WCS_DISTORTION_TPD;
-      else if( !strcmp(*disseq->dtype, "DSS") ) return GAL_WCS_DISTORTION_DSS;
-      else if( !strcmp(*disseq->dtype, "WAT") ) return GAL_WCS_DISTORTION_WAT;
+      if(      !strcmp(*disseq->dtype, "TPV") )
+        return GAL_WCS_DISTORTION_TPV;
+      else if( !strcmp(*disseq->dtype, "TPD") )
+        return GAL_WCS_DISTORTION_TPD;
+      else if( !strcmp(*disseq->dtype, "DSS") )
+        return GAL_WCS_DISTORTION_DSS;
+      else if( !strcmp(*disseq->dtype, "WAT") )
+        return GAL_WCS_DISTORTION_WAT;
       else
-        error(EXIT_FAILURE, 0, "%s: distortion '%s' isn't recognized in "
-              "the 'disseq' structure of the given 'wcsprm'", __func__,
-              *dispre->dtype);
+        return GAL_WCS_DISTORTION_INVALID;
     }
 
   /* Control should not reach here. */
@@ -1607,6 +1639,11 @@ gal_wcs_warp_matrix(struct wcsprm *wcs)
       out[1] = -1 * wcs->cdelt[1] *sin(crota2);
       out[2] = wcs->cdelt[0] * sin(crota2);
       out[3] = wcs->cdelt[1] * cos(crota2);
+
+      /* For a check:
+      printf("cdelt: %f, %f\n", wcs->cdelt[0], wcs->cdelt[1]);
+      printf("cd:\n%f, %f\n%f, %f\n", out[0], out[1], out[2], out[3]);
+      */
     }
   else
     error(EXIT_FAILURE, 0, "%s: currently only PCi_ja and CDi_ja keywords "
@@ -1663,8 +1700,7 @@ gal_wcs_clean_errors(struct wcsprm *wcs)
 /* According to the FITS standard, in the 'PCi_j' WCS formalism, the matrix
    elements m_{ij} are encoded in the 'PCi_j' keywords and the scale
    factors are encoded in the 'CDELTi' keywords. There is also another
-   formalism (the 'CDi_j' formalism) which merges the two into one
-   matrix.
+   formalism (the 'CDi_j' formalism) which merges the two into one matrix.
 
    However, WCSLIB's internal operations are apparently done in the 'PCi_j'
    formalism. So its outputs are also all in that format by default. When
@@ -1676,6 +1712,7 @@ void
 gal_wcs_decompose_pc_cdelt(struct wcsprm *wcs)
 {
   size_t i, j;
+  int status=0;
   double *ps, *warp;
 
   /* If there is on WCS, then don't do anything. */
@@ -1687,7 +1724,7 @@ gal_wcs_decompose_pc_cdelt(struct wcsprm *wcs)
   if(ps==NULL) return;
 
   /* For a check.
-  printf("pc: %g, %g\n", pc[0], pc[1]);
+  printf("ps: %g, %g\n", ps[0], ps[1]);
   printf("warp: %g, %g, %g, %g\n", warp[0], warp[1],
          warp[2], warp[3]);
   */
@@ -1710,6 +1747,12 @@ gal_wcs_decompose_pc_cdelt(struct wcsprm *wcs)
      assumptioins about 'CDELTi'. */
   wcs->altlin=1;
 
+  /* Re-run 'wcsset' to update all the internal WCS parameters. */
+  status=wcsset(wcs);
+  if(status)
+    error(EXIT_SUCCESS, 0, "%s: WCSLIB warning: wcsset ERROR %d: %s",
+          __func__, status, wcs_errmsg[status]);
+
   /* Clean up. */
   free(ps);
   free(warp);
@@ -1997,9 +2040,10 @@ gal_wcs_coverage(char *filename, char *hdu, size_t *ondim,
   struct wcsprm *wcs;
   int nwcs=0, type, status=0;
   char *name=NULL, *unit=NULL;
-  gal_data_t *tmp, *coords=NULL;
+  gal_data_t *fc, *tmp, *coords=NULL;
   size_t i, ndim, *dsize=NULL, numrows;
-  double *x=NULL, *y=NULL, *z=NULL, *min, *max, *center, *width;
+  size_t fullcircledim=GAL_BLANK_SIZE_T;
+  double *x=NULL, *y=NULL, *z=NULL, *min, *max, *fcarr, *center, *width;
 
   /* Read the desired WCS (note that the linear matrix is irrelevant here,
      we'll just select PC because its the default WCS mode. */
@@ -2016,7 +2060,7 @@ gal_wcs_coverage(char *filename, char *hdu, size_t *ondim,
           filename, hdu);
 
   /* Get the array information of the image. */
-  fptr=gal_fits_hdu_open(filename, hdu, READONLY);
+  fptr=gal_fits_hdu_open(filename, hdu, READONLY, 1);
   gal_fits_img_info(fptr, &type, ondim, &dsize, &name, &unit);
   fits_close_file(fptr, &status);
   ndim=*ondim;
@@ -2118,25 +2162,79 @@ gal_wcs_coverage(char *filename, char *hdu, size_t *ondim,
     }
 
   /* Write the center and width. */
+  width[0]=max[0]-min[0];
+  width[1]=max[1]-min[1];
   switch(ndim)
     {
     case 2:
-      center[0]=x[4];         center[1]=y[4];
-      width[0]=max[0]-min[0]; width[1]=max[1]-min[1];
+      center[0]=x[4];  center[1]=y[4];
       break;
     case 3:
-      center[0]=x[8];         center[1]=y[8];         center[2]=z[8];
-      width[0]=max[0]-min[0]; width[1]=max[1]-min[1]; width[2]=max[2]-min[2];
+      width[2]=max[2]-min[2];
+      center[0]=x[8]; center[1]=y[8]; center[2]=z[8];
       break;
     default:
-      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to solve the "
-            "problem. The value %zu is not a recognized dimension", __func__,
-            PACKAGE_BUGREPORT, ndim);
+      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to solve "
+            "the problem. The value %zu is not a recognized dimension",
+            __func__, PACKAGE_BUGREPORT, ndim);
+    }
+
+  /* It may happen that the image passes the RA=0 hour circle. In that
+     case, the width along the first WCS dimension (RA) will be larger than
+     180. Therefore, we need to re-calculate the minimum and maximums. An
+     example dataset is available in https://savannah.gnu.org/bugs/?63257
+     (Gnuastro bug #63257):
+
+        $ jplusdr2url=http://archive.cefca.es/catalogues/vo/siap/jplus-dr2
+        $ wget $jplusdr2url/get_fits?id=71811 -Ojplus-dr2-71811.fits.fz
+
+     But first, we need to find the dimension that has a full-circle
+     dimension: this is usually the first WCS dimension, but it may happen
+     that it isn't. Also, note that the WCS may not be celestial/spherical
+     at all (hence why we are only doing this for pre-defined celestial
+     'CTYPE's).*/
+  for(i=0;i<ndim;++i)
+    if(   strncmp(wcs->ctype[i], "RA-",   3)==0 /* Equatorial */
+       || strncmp(wcs->ctype[i], "GLON-", 5)==0 /* Galactic */
+       || strncmp(wcs->ctype[i], "SLON-", 5)==0 /* Super Galactic */
+       || strncmp(wcs->ctype[i], "ELON-", 5)==0 /* Ecliptic */ )
+      { fullcircledim=i; break; }
+  if( fullcircledim!=GAL_BLANK_SIZE_T && width[fullcircledim]>180.0f )
+    {
+      /* Set the proper pointer to the coordinate that should be
+         checked. */
+      switch(fullcircledim)
+        {
+        case 0: fc=coords;             break;
+        case 1: fc=coords->next;       break;
+        case 2: fc=coords->next->next; break;
+        default:
+          error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at "
+                "'%s' to fix the problem. The value '%zu' isn't "
+                "recognized for 'fullcircledim'", __func__,
+                PACKAGE_BUGREPORT, fullcircledim);
+        }
+
+      /* Shift all the coordinates after 0, by 360, while keeping those
+         that are close to 360 untouched, then find the minimum and
+         maximums. */
+      fcarr=fc->array;
+      for(i=0;i<coords->size;++i) if(fcarr[i]<180) fcarr[i]+=360.0f;
+      tmp=gal_statistics_minimum(fc);
+      min[fullcircledim] = ((double *)(tmp->array))[0]; gal_data_free(tmp);
+      tmp=gal_statistics_maximum(fc);
+      max[fullcircledim] = ((double *)(tmp->array))[0]; gal_data_free(tmp);
+
+      /* Calculate the correct width, maximum is guarateed to be larger
+         than 360, so subtract 360 from it. */
+      width[fullcircledim]=max[fullcircledim]-min[fullcircledim];
+      max[fullcircledim]-=360.0f;
     }
 
   /* Clean up and return success. */
   free(dsize);
-  wcsfree(wcs);
+  wcsfree(wcs); free(wcs);
+  gal_list_data_free(coords);
   return 1;
 }
 
@@ -2211,7 +2309,7 @@ wcs_convert_sanity_check_alloc(gal_data_t *coords, struct wcsprm *wcs,
   /* Allocate all the necessary arrays. */
   *phi    = gal_pointer_allocate( GAL_TYPE_FLOAT64, size,      0, __func__,
                                   "phi");
-  *stat   = gal_pointer_allocate( GAL_TYPE_INT32,   size,      1, __func__,
+  *stat   = gal_pointer_allocate( GAL_TYPE_INT,     size,      1, __func__,
                                   "stat");
   *theta  = gal_pointer_allocate( GAL_TYPE_FLOAT64, size,      0, __func__,
                                   "theta");
@@ -2232,8 +2330,8 @@ wcs_convert_sanity_check_alloc(gal_data_t *coords, struct wcsprm *wcs,
    list. But in WCSLIB, the input is a single array (with multiple
    columns). This function will convert between the two. */
 static void
-wcs_convert_list_to_array(gal_data_t *list, double *array, int *stat,
-                          size_t ndim, int listtoarray)
+wcs_convert_list_to_from_array(gal_data_t *list, double *array, int *stat,
+                               size_t ndim, int to0from1)
 {
   size_t i, d=0;
   gal_data_t *tmp;
@@ -2244,10 +2342,10 @@ wcs_convert_list_to_array(gal_data_t *list, double *array, int *stat,
          input into or output from WCSLIB. */
       for(i=0;i<list->size;++i)
         {
-          if(listtoarray)
-            array[i*ndim+d] = ((double *)(tmp->array))[i];
-          else
+          if(to0from1)
             ((double *)(tmp->array))[i] = stat[i] ? NAN : array[i*ndim+d];
+          else
+            array[i*ndim+d] = ((double *)(tmp->array))[i];
         }
 
       /* Increment the dimension. */
@@ -2265,14 +2363,15 @@ wcs_convert_prepare_out(gal_data_t *coords, struct wcsprm *wcs, int inplace)
 {
   size_t i;
   gal_data_t *out=NULL;
+
   if(inplace)
     out=coords;
   else
     for(i=0;i<wcs->naxis;++i)
       gal_list_data_add_alloc(&out, NULL, GAL_TYPE_FLOAT64, 1,
                               &coords->size, NULL, 0, coords->minmapsize,
-                              coords->quietmmap, wcs->ctype[i], wcs->cunit[i],
-                              NULL);
+                              coords->quietmmap, wcs->ctype[i],
+                              wcs->cunit[i], NULL);
   return out;
 }
 
@@ -2289,25 +2388,35 @@ gal_data_t *
 gal_wcs_world_to_img(gal_data_t *coords, struct wcsprm *wcs, int inplace)
 {
   gal_data_t *out;
-  int status, *stat=NULL, ncoord=coords->size, nelem;
+  int *stat=NULL, ncoord=coords->size, nelem;
   double *phi=NULL, *theta=NULL, *world=NULL, *pixcrd=NULL, *imgcrd=NULL;
 
+  /* It can happen that the input datasets are empty. In this case, simply
+     return them. */
+  if(coords->size==0 || coords->array==NULL)
+    {
+      if(inplace) return coords;
+      else error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at "
+                 "'%s' to fix the problem. The input has no data and "
+                 "'inplace' is not called", __func__, PACKAGE_BUGREPORT);
+    }
+
+
   /* Some sanity checks. */
-  wcs_convert_sanity_check_alloc(coords, wcs, __func__, &stat, &phi, &theta,
-                                 &world, &pixcrd, &imgcrd);
+  wcs_convert_sanity_check_alloc(coords, wcs, __func__, &stat, &phi,
+                                 &theta, &world, &pixcrd, &imgcrd);
   nelem=wcs->naxis; /* We have to make sure a WCS is given first. */
 
 
   /* Write the values from the input list of separate columns into a single
      array (WCSLIB input). */
-  wcs_convert_list_to_array(coords, world, stat, wcs->naxis, 1);
+  wcs_convert_list_to_from_array(coords, world, stat, wcs->naxis, 0);
 
 
-  /* Use WCSLIB's wcss2p for the conversion. */
-  status=wcss2p(wcs, ncoord, nelem, world, phi, theta, imgcrd, pixcrd, stat);
-  if(status)
-    error(EXIT_FAILURE, 0, "%s: wcss2p ERROR %d: %s", __func__, status,
-          wcs_errmsg[status]);
+  /* Use WCSLIB's wcss2p for the conversion. We are ignoring the over-all
+     status here, because later we will use the 'stat' array to set all bad
+     coordinates to NaN. */
+  wcss2p(wcs, ncoord, nelem, world, phi, theta, imgcrd, pixcrd, stat);
 
 
   /* For a sanity check.
@@ -2315,9 +2424,13 @@ gal_wcs_world_to_img(gal_data_t *coords, struct wcsprm *wcs, int inplace)
     size_t i;
     printf("\n\n%s sanity check:\n", __func__);
     for(i=0;i<coords->size;++i)
-      printf("(%g, %g, %g) --> (%g, %g, %g), [stat: %d]\n",
-              world[i*3],  world[i*3+1 ], world[i*3+2],
-             pixcrd[i*3], pixcrd[i*3+1], pixcrd[i*3+2], stat[i]);
+      printf("(%g, %g) --> phi:%g, theta: %g --> (%g, %g) --> (%g, %g) "
+             "[stat: %d]\n",
+             world[i*2],  world[i*2+1],
+             phi[i*2],    theta[i*2],
+             imgcrd[i*2], imgcrd[i*2+1],
+             pixcrd[i*2], pixcrd[i*2+1], stat[i]);
+    printf("stat: %d\n", stat[4]);
   }
   */
 
@@ -2328,7 +2441,7 @@ gal_wcs_world_to_img(gal_data_t *coords, struct wcsprm *wcs, int inplace)
 
   /* Write the output from a single array (WCSLIB output) into the output
      list of this function. */
-  wcs_convert_list_to_array(out, pixcrd, stat, wcs->naxis, 0);
+  wcs_convert_list_to_from_array(out, pixcrd, stat, wcs->naxis, 1);
 
 
   /* Clean up. */
@@ -2336,6 +2449,7 @@ gal_wcs_world_to_img(gal_data_t *coords, struct wcsprm *wcs, int inplace)
   free(stat);
   free(theta);
   free(world);
+  free(imgcrd);
   free(pixcrd);
 
   /* Return the output list of coordinates. */
@@ -2351,7 +2465,7 @@ gal_data_t *
 gal_wcs_img_to_world(gal_data_t *coords, struct wcsprm *wcs, int inplace)
 {
   gal_data_t *out;
-  int status, *stat=NULL, ncoord=coords->size, nelem;
+  int *stat=NULL, ncoord=coords->size, nelem;
   double *phi=NULL, *theta=NULL, *world=NULL, *pixcrd=NULL, *imgcrd=NULL;
 
   /* Some sanity checks. */
@@ -2362,14 +2476,13 @@ gal_wcs_img_to_world(gal_data_t *coords, struct wcsprm *wcs, int inplace)
 
   /* Write the values from the input list of separate columns into a single
      array (WCSLIB input). */
-  wcs_convert_list_to_array(coords, pixcrd, stat, wcs->naxis, 1);
+  wcs_convert_list_to_from_array(coords, pixcrd, stat, wcs->naxis, 0);
 
 
-  /* Use WCSLIB's wcsp2s for the conversion. */
-  status=wcsp2s(wcs, ncoord, nelem, pixcrd, imgcrd, phi, theta, world, stat);
-  if(status)
-    error(EXIT_FAILURE, 0, "%s: wcsp2s ERROR %d: %s", __func__, status,
-          wcs_errmsg[status]);
+  /* Use WCSLIB's wcsp2s for the conversion. We are ignoring the over-all
+     status here, because later we will use the 'stat' array to set all bad
+     coordinates to NaN.*/
+  wcsp2s(wcs, ncoord, nelem, pixcrd, imgcrd, phi, theta, world, stat);
 
 
   /* For a check.
@@ -2400,7 +2513,7 @@ gal_wcs_img_to_world(gal_data_t *coords, struct wcsprm *wcs, int inplace)
 
   /* Write the output from a single array (WCSLIB output) into the output
      list of this function. */
-  wcs_convert_list_to_array(out, world, stat, wcs->naxis, 0);
+  wcs_convert_list_to_from_array(out, world, stat, wcs->naxis, 1);
 
 
   /* Clean up. */
@@ -2408,6 +2521,7 @@ gal_wcs_img_to_world(gal_data_t *coords, struct wcsprm *wcs, int inplace)
   free(stat);
   free(theta);
   free(world);
+  free(imgcrd);
   free(pixcrd);
 
 
